//
// vfive: Vector Field Interactive Visualization Environment
//
//
// boundary_box.c++
//
//    Region boundary denoted by lines
//
//                by Kageyama, Akira  (kage@nifs.ac.jp)  2000.03.18
//  
// Please read the file "LICENCE_ja.txt"(in Japanese) or "LICENCE_en.txt" (in English)
// before you use this software. 

#include "vfive.h"

extern Data* data;

BoundaryBox:: BoundaryBox()
{
  on = 1;               // default on
}

void
BoundaryBox:: initialize()
{
  int tID = CAVEUniqueIndex();

  dislis[tID] = glGenLists(1);
  glNewList(dislis[tID], GL_COMPILE);
  glDisable(GL_LIGHTING);
  glLineWidth(1.2);

  int    i, j, k;
  double x, y, z;

  int n1 = data->n1;
  int n2 = data->n2;
  int n3 = data->n3;
  
  //
  // Line along the 1st coordinate 
  //
  glColor3f(1.0, 0.0, 0.0);
  for (j = 0; j < n2; j += n2 - 1) {
    for (k = 0; k < n3; k += n3 - 1) {
      y = data->y[j];
      z = data->z[k];
      glBegin(GL_LINES);
        glVertex3f(data->region_xmin, y, z);
        glVertex3f(data->region_xmax, y, z);
      glEnd();
    }
  }

  //
  // Line along the 2nd coordinate 
  //
  glColor3f(0.0, 1.0, 0.0);
  for (i = 0; i < n1; i += n1 - 1) {
    for (k = 0; k < n3; k += n3 - 1) {
      x = data->x[i];
      z = data->z[k];
      glBegin(GL_LINES);
        glVertex3f(x, data->region_ymin, z);
        glVertex3f(x, data->region_ymax, z);
      glEnd();
    }
  }

  //
  // Line along the 3rd coordinate
  //
  glColor3f(0.0, 0.0, 1.0);
  for (i = 0; i < n1; i += n1 - 1) {
    for (j = 0; j < n2; j += n2 - 1) {
      x = data->x[i];
      y = data->y[j];
      glBegin(GL_LINES);
        glVertex3f(x, y, data->region_zmin);
        glVertex3f(x, y, data->region_zmax);
      glEnd();
    }
  }
  glEndList();
}

void
BoundaryBox:: draw()
{
  if (on==0) return;
  glCallList(dislis[CAVEUniqueIndex()]);
}
