//
// vfive: Vector Field Interactive Visualization Environment
//
//
// curve_obj.c++
//
//                by Kageyama, Akira  (kage@nifs.ac.jp)  1999.11.29
//                                     C to C++ by kage  2000.03.13
//
//                                                 kage  2003.12.18
//  
// Please read the file "LICENCE_ja.txt"(in Japanese) or "LICENCE_en.txt" (in English)
// before you use this software. 


#include "vfive.h"

extern Data* data;

const float BIG_VALUE_FOR_EDGE_CHECK = 999.0;

static void 
read_data(char *file, float *a, int size)  
{
    FILE *fp;
    char dummy[4];

    fp = fopen(file,"rb");
    if (fp==NULL) v5_error("fopen failed in readata, read_curveObj.c++");
    fread(dummy, 1, 4, fp); // skip the first 4 byte in a fortran record
    fread((char *)a, 1, size*sizeof(float), fp);
    fclose(fp);
}


void
CurveObj:: read()
{
  char xfile[200], yfile[200], zfile[200], cfile[200];

   v5_get_string(inputFile, "curveObj:xfile", xfile);
   v5_get_string(inputFile, "curveObj:yfile", yfile);
   v5_get_string(inputFile, "curveObj:zfile", zfile);
   v5_get_string(inputFile, "curveObj:cfile", cfile);

   // check the results 
   printf("   ntenmax = %d\n", ntenmax);
   printf("   nhon    = %d\n", nhon);
   printf("   xfile   = %s\n", xfile);
   printf("   yfile   = %s\n", yfile);
   printf("   zfile   = %s\n", zfile);
   printf("   cfile   = %s\n", cfile);


  // data rading 
  read_data(xfile, &(x[0][0]), nhon*ntenmax);
  read_data(yfile, &(y[0][0]), nhon*ntenmax);
  read_data(zfile, &(z[0][0]), nhon*ntenmax);
  read_data(cfile, &(c[0][0]), nhon*ntenmax);

  const double feet_per_meter = 3.280840;
  double norms = feet_per_meter * (data->scale);
  double shift_x = data->shift_x;
  double shift_y = data->shift_y;
  double shift_z = data->shift_z;

  for (int i=0; i<nhon; i++) {
    int n = 0;
    for (int j=0; j<ntenmax; j++) {
      float x_ = x[i][j];
      float y_ = y[i][j];
      float z_ = z[i][j];
      if   ( x_ > BIG_VALUE_FOR_EDGE_CHECK 
          && y_ > BIG_VALUE_FOR_EDGE_CHECK 
          && z_ > BIG_VALUE_FOR_EDGE_CHECK ) break;
      x[i][j] += shift_x;        // coordinate shift 
      y[i][j] += shift_y;
      z[i][j] += shift_z;
      x[i][j] *= norms;          // coordinate scale
      y[i][j] *= norms;          // coordinate scale
      z[i][j] *= norms;          // coordinate scale
      ++n;
    }
    nten[i] = n;
  }
}


void
CurveObj:: make(char* file)
{
  inputFile = file;
  v5_get_int(file, "curveObj:nhon", &nhon);
  v5_get_int(file, "curveObj:ntenmax", &ntenmax);

  printf("debug: curveObj: nhon = %d ntenmax = %d\n", nhon, ntenmax);
  x = create_array_f2(nhon, ntenmax);
  y = create_array_f2(nhon, ntenmax);
  z = create_array_f2(nhon, ntenmax);
  c = create_array_f2(nhon, ntenmax);
  nten = create_array_i1(nhon);
  turn(0);                   // default off
}

void
CurveObj:: initialize()
{
  int tID = CAVEUniqueIndex();
  dislis[tID] = glGenLists(1);
  glNewList(dislis[tID], GL_COMPILE);

/******  No tube ******
  glEnable(GL_LIGHTING);
  glMaterialfv(GL_FRONT, GL_DIFFUSE,  diffuse);
  glMaterialfv(GL_FRONT, GL_AMBIENT,  ambient);
  glMaterialfv(GL_FRONT, GL_SPECULAR, specular);
  glMaterialf (GL_FRONT, GL_SHININESS, 10.0);
  for (int i = 0; i < nhon; i++) {
    if (nten[i] == 0) continue;
    tube_construct(x[i], y[i],z[i], nten[i]);
  }
******/

// Added 2003.05.29 by Akira Kageyama, ESC
// debug (line end) 2003.12.18 by Akira Kageyama, ESC
  glDisable(GL_LIGHTING);
  glColor3f(0.5, 0.5, 0.5);
  glLineWidth(1.0);
  glPushMatrix();
  for (int i=0; i<nhon; i++) {
    glBegin(GL_LINE_STRIP);
      for (int j=0; j<nten[i]; j++) {
        glVertex3f(x[i][j],y[i][j],z[i][j]);
      }
    glEnd();
  }
  glPopMatrix();

  glEndList();
}

void
CurveObj:: draw()
{
  if (!on) return;

  glPushAttrib(GL_LIGHTING_BIT);
  glCallList(dislis[CAVEUniqueIndex()]);
  glPopAttrib();
}


