//
// vfive: Vector Field Interactive Visualization Environment
//
// data.c++
//
//                by Kageyama, Akira  (kage@nifs.ac.jp)  2000.03.17
//  
// Please read the file "LICENCE_ja.txt"(in Japanese) or "LICENCE_en.txt" (in English)
// before you use this software. 

#include "vfive.h"

extern Data* data;

static const float feet_per_meter = 3.280840;

Data::Data(char *inputFile_)
{
  int n;

  strcpy(inputFile, inputFile_);
  v5_get_int(inputFile, "n1", &n1);  
  v5_get_int(inputFile, "n2", &n2);
  v5_get_int(inputFile, "n3", &n3);
  v5_get_int(inputFile, "nvec", &nvec);
  v5_get_int(inputFile, "nscal", &nscal);
  v5_get_float(inputFile, "scale",   &scale);
  v5_get_float(inputFile, "shift_x", &shift_x);
  v5_get_float(inputFile, "shift_y", &shift_y);
  v5_get_float(inputFile, "shift_z", &shift_z);


  x = create_array_ff1(n1);     //  3D array 
  y = create_array_ff1(n2);     //  of field float.
  z = create_array_ff1(n3);

  //  Vector fields 
  for ( n = 0; n < nvec; n++) {
    // debug s
    printf("  vector n = %d\n", n);
    // debug e
    vector_fld[n] = new Field_v;
     if(vector_fld[n] == NULL) v5_error("can't allocate vector_fld: data");
    vector_fld[n]->x = create_array_ff3(n3, n2, n1); //  3D array 
    // debug s
    puts(" ----- x done.");
    // debug e
    vector_fld[n]->y = create_array_ff3(n3, n2, n1);
    // debug s
    puts(" ----- y done.");
    // debug e
    vector_fld[n]->z = create_array_ff3(n3, n2, n1);
    // debug s
    puts(" ----- z done.");
    // debug e
  }

  //  Scalar fields
  for (n = 0; n < nscal; n++) {
    // debug s
    printf("  scalar n = %d\n", n);
    // debug e
    scalar_fld[n] = new Field_s;
     if(scalar_fld[n] == NULL) v5_error("can't allocate scalar_fld: data");
    scalar_fld[n]->val = create_array_ff3(n3, n2, n1);
  }
}

static void
read_coord(char compo) {
  FILE *fp;
  char dummy[4], key[50], coord_file[200];
   
  int n1 = data->n1;
  int n2 = data->n2;
  int n3 = data->n3;

   // Only Master Node reads data file
   sprintf(key, "%cfile", compo);
   v5_get_string(data->inputFile, key, coord_file);

   fp = fopen(coord_file,"rb");
   if (fp == NULL) v5_error(" Open failed. read_coordinates.");
   fread(dummy, 1, 4, fp);

   if ( compo=='x' ){ 
     fread((char *)data->x, 1, n1*sizeof(ffloat_), fp);
    } else if ( compo=='y' ){
     fread((char *)data->y, 1, n2*sizeof(ffloat_), fp);
   } else if ( compo=='z' ){
     fread((char *)data->z, 1, n3*sizeof(ffloat_), fp);
    } else 
     v5_error(" Bad compo! read_coordinates.");
  
   fclose(fp);


}

static void
coord_trans()
{
  int i, j, k; 

  float norms = feet_per_meter * (data->scale);
  printf("  norms = %f\n", norms);
  
  //-- coordinate transform 
  for(i=0; i<data->n1; i++) data->x[i] += data->shift_x;
  for(j=0; j<data->n2; j++) data->y[j] += data->shift_y;
  for(k=0; k<data->n3; k++) data->z[k] += data->shift_z;

  //--- coordinate scaling so that unit 1 means 1 meter
  for(i=0; i<data->n1; i++) data->x[i] *= norms;
  for(j=0; j<data->n2; j++) data->y[j] *= norms;
  for(k=0; k<data->n3; k++) data->z[k] *= norms;
}


static void 
read_vect(int nthvec, char compo)  
{
  FILE *fp;
  char dummy[4], vect_file[200], key[50];
    
  int n123 = (data->n1)*(data->n2)*(data->n3);
  
   // Only Master Node reads data file
   sprintf(key, "vect%d%c", nthvec, compo);
   v5_get_string(data->inputFile, key, vect_file);
   printf("v5: vect_file = %s\n", vect_file);
   fp = fopen(vect_file,"rb");
   if (fp == NULL) v5_error(" Open failed. read_vectors");

   /*  NO SKIP. JUST FOR TEMPORARY.  080526 */
   fread(dummy, 1, 4, fp);

  
   if ( compo=='x' ){
     fread((char *)(data->vector_fld[nthvec]->x[0][0]), 
	  1, n123*sizeof(ffloat_), fp);
   } else if ( compo=='y' ){
     fread((char *)(data->vector_fld[nthvec]->y[0][0]), 
	  1, n123*sizeof(ffloat_), fp);
   } else if ( compo=='z' ){
     fread((char *)(data->vector_fld[nthvec]->z[0][0]), 
	  1, n123*sizeof(ffloat_), fp);
   } else
     v5_error("*** Bad compo! read_vectors.");

   fclose(fp);
}


static void 
read_scalar(int nth)  
{
  FILE *fp;
  char dummy[4], scal_file[200], key[50];

  if ( nth>data->nscal ) v5_error(" bad nth. read_scalars");
  
  int n123 = (data->n1)*(data->n2)*(data->n3);

   // Only Master Node reads data file
  sprintf(key, "scal%d", nth);
  v5_get_string(data->inputFile, key, scal_file);
  fp = fopen(scal_file,"rb");
  if (fp==NULL) v5_error(" Open failed; read_scalars.");

  /* NO SKIP. JUST FOR TEMPORARY*/
  fread(dummy, 1, 4, fp);

  fread((char *)(data->scalar_fld[nth]->val[0][0]), 
	1, n123*sizeof(ffloat_), fp);
  fclose(fp);

}


static void 
read()
{
  read_coord('x');
  read_coord('y');
  read_coord('z');

  for (int i=0; i<data->nvec; i++) {
    read_vect(i,'x');
    read_vect(i,'y');
    read_vect(i,'z');
  }

  for (int n=0; n<data->nscal; n++)  
    read_scalar(n);

  coord_trans();
}

static void
set_grid_size_min() 
{
  //  To find minimum grid size to estimate "dt" of the RK integration
  int   i,j,k;
  ffloat_ dxmin = 1.e20; /* something very large */
  ffloat_ dymin = 1.e20;
  ffloat_ dzmin = 1.e20;
  ffloat_ dx, dy, dz, dd;

  int n1 = data->n1;
  int n2 = data->n2;
  int n3 = data->n3;

  for (i=0; i<n1-1; i++) {
    dx = data->x[i+1]-data->x[i];
    if (dx < 0) {
      v5_error("   x[i] must increase as i.");
      }
    if (dx < dxmin) dxmin = dx;
  }
  for (j=0; j<n2-1; j++) {
    dy = data->y[j+1]-data->y[j];
    if (dy < 0) {
      v5_error("   y[j] must increase as j.");
      }
    if (dy < dymin) dymin = dy;
  }
  for (k=0; k<n3-1; k++) {
    dz = data->z[k+1]-data->z[k];
    if (dz < 0) {
      v5_error("   z[k] must increase as k.");
      }
    if (dz < dzmin) dzmin = dz;
  }
  dd = dxmin < dymin ? dxmin : dymin;
  data->grid_size_min = (dd < dzmin ? dd : dzmin);
}
  

static void
coord_check()
{
  
  data->region_xmin = data->x[0];
  data->region_xmax = data->x[data->n1-1];
  data->region_ymin = data->y[0];
  data->region_ymax = data->y[data->n2-1];
  data->region_zmin = data->z[0];
  data->region_zmax = data->z[data->n3-1];

  printf("v5: coordinate xmin = %f\n", data->region_xmin);
  printf("v5: coordinate xmax = %f\n", data->region_xmax);
  printf("v5: coordinate ymin = %f\n", data->region_ymin);
  printf("v5: coordinate ymax = %f\n", data->region_ymax);
  printf("v5: coordinate zmin = %f\n", data->region_zmin);
  printf("v5: coordinate zmax = %f\n", data->region_zmax);

  if (data->region_xmin >= data->region_xmax || 
      data->region_ymin >= data->region_ymax || 
      data->region_zmin >= data->region_zmax)  {
    v5_error(" Something in coordinates is strnage.\n");
  }
}

static void
vect_check()
{    
  int n1 = data->n1;
  int n2 = data->n2;
  int n3 = data->n3;

  for(int n=0; n<data->nvec; n++) {

    ffloat_ vmin =  1.e20;
    ffloat_ vmax = -1.e20;

    for (int i=0; i<n1; i++) {
      for (int j=0; j<n2; j++) {
	for (int k=0; k<n3; k++) {
	  ffloat_ vx = (data->vector_fld[n])->x[k][j][i];
	  ffloat_ vy = (data->vector_fld[n])->y[k][j][i];
	  ffloat_ vz = (data->vector_fld[n])->z[k][j][i];
	  ffloat_ vv = sqrt(vx*vx+vy*vy+vz*vz);

	  if(vv > vmax) vmax = vv;
	  if(vv < vmin) vmin = vv;

	}
      }
    }

    data->vector_fld[n]->vmin = vmin;
    data->vector_fld[n]->vmax = vmax;

    printf("v5: %d th vec: vmin = %f vmax = %f\n",n, vmin, vmax);
  }
}
static void 
scalar_check()  
{
  int i,j,k,n,ip,im,jp,jm,kp,km;
  ffloat_ vmin, vmax, vv;
  ffloat_ vol, dvol, avr, var;
  ffloat_ dx,dy,dz;

  int n1 = data->n1;
  int n2 = data->n2;
  int n3 = data->n3;

  for(n=0; n<data->nscal; n++) {
    vmin =  1.e20;
    vmax = -1.e20;
    avr = 0.0;
    var = 0.0;
    vol = 0.0;
    for (i=0; i<n1; i++) {
      if     (i==0)    { ip = 1;    im = 0; }
      else if(i==n1-1) { ip = n1-1; im = n1-2; }
      else             { ip = i+1;  im = i-1; }
      dx = (data->x[ip] - data->x[im])*0.5;

      for (j=0; j<n2; j++) {
	if     (j==0)    { jp = 1;    jm = 0; }
	else if(j==n2-1) { jp = n2-1; jm = n2-2; }
	else             { jp = j+1;  jm = j-1; }
	dy = (data->y[jp] - data->y[jm])*0.5;

	for (k=0; k<n3; k++) {
	  if     (k==0)    { kp = 1;    km = 0; }
	  else if(k==n3-1) { kp = n3-1; km = n3-2; }
	  else             { kp = k+1;  km = k-1; }
	  dz = (data->z[kp] - data->z[km])*0.5;
	  
	  dvol = dx*dy*dz;
	  vv = data->scalar_fld[n]->val[k][j][i];

	  if(vv>vmax) vmax = vv;
	  if(vv<vmin) vmin = vv;

//		if(vv > 0.0 || vv < 0.0) printf("%f",vv);

	  vol += dvol;
	  avr += vv*dvol;
	  var += (vv*vv)*dvol;
	}
      }
    }
    avr /= vol;
    var /= vol;

    data->scalar_fld[n]->vmin = vmin;
    data->scalar_fld[n]->vmax = vmax;
    data->scalar_fld[n]->aver = avr;
    data->scalar_fld[n]->vari = sqrt(var - avr*avr);
    printf("v5:  Scalar field: vmin = %f\n", vmin);
    printf("v5:                vmax = %f\n", vmax);
    printf("v5:                aver = %f\n", avr);
    printf("v5:                vari = %f\n", sqrt(var - avr*avr));
  }
}

void
Data::set()
{
  read();
  coord_check();
  vect_check();
  scalar_check();
  set_grid_size_min();
}
