//
// vfive: Vector Field Interactive Visualization Environment
//
// hotaru.c++
//
//                by Kageyama, Akira  (kage@nifs.ac.jp)  2000.04.18
//  
// Please read the file "LICENCE_ja.txt"(in Japanese) or "LICENCE_en.txt" (in English)
// before you use this software. 


#include "vfive.h"

extern Data *data;

Hotaru::Hotaru()
{
 on = 0;
 svec_ = 0;
  // From initial
  xmin = data->region_xmin;
  xmax = data->region_xmax;
  ymin = data->region_ymin;
  ymax = data->region_ymax;
  zmin = data->region_zmin;
  zmax = data->region_zmax;
  xlen = xmax - xmin;
  ylen = ymax - ymin;
  zlen = zmax - zmin;

  for(int i=0;i<HOTARU_NUM;i++){
   hotaru_pos[i][0] = 0.0;      // x,y,z
   hotaru_pos[i][1] = 0.0;      // x,y,z
   hotaru_pos[i][2] = 0.0;      // x,y,z
  }

}

static double random0to1(){

    const int N1=1229, N2=351750, N3=1664501;
    static int nrandm = 229933;//any integer
    double arandm;

    nrandm = (N1*nrandm+N2)%N3;
    arandm = (double)nrandm/(double)N3;

	return arandm;
}


int
Hotaru::is_in_the_region(float *pos)
{
  //
  // Check if the 'hotaru' is inside (1) of the simulation
  // region, or not (0).
  //
// 2008.12
//  if (   pos[0] >= xmin && pos[0] <= xmax
//     && pos[1] >= ymin && pos[1] <= ymax
//      && pos[2] >= zmin && pos[2] <= zmax ) 
  if (   pos[0] >= xmin && pos[0] < xmax
      && pos[1] >= ymin && pos[1] < ymax
      && pos[2] >= zmin && pos[2] < zmax ) 
    return 1;                           // In the region.

  return 0;                             // Outside of the spot.
}


void
Hotaru::reposit_hotaru_i(int i)
  // 
  // Place a hotaru.
  //
{
  hotaru_pos[i][0] = xmin + (float)random0to1() * xlen;
  hotaru_pos[i][1] = ymin + (float)random0to1() * ylen;
  hotaru_pos[i][2] = zmin + (float)random0to1() * zlen;
}

void
Hotaru::place_hotaru()
  // 
  // Place all hotarus.
  //
{
  for (int i=0; i<HOTARU_NUM; i++) 
    reposit_hotaru_i(i);
}

void
Hotaru::initialize() 
{
}


void
Hotaru::integrate()
{
  if (!on) return;
  
  int id;
  float vec[3], new_pos[3];
  float val;
  int sv = svec();

  float dt = 0.1 / data->vector_fld[sv]->vmax;
  
  for (int i=0; i<HOTARU_NUM; i++) {
    // Interpolation function is common with snow.
    id = intpol_get_vect_for_snow(hotaru_pos[i], vec, &val);
    if (id) {
      reposit_hotaru_i(i);
      continue;
    }
    new_pos[0] = hotaru_pos[i][0] + vec[0] * dt;
    new_pos[1] = hotaru_pos[i][1] + vec[1] * dt;
    new_pos[2] = hotaru_pos[i][2] + vec[2] * dt;
    if (is_in_the_region(new_pos) ) {
      hotaru_pos[i][0] = new_pos[0];
      hotaru_pos[i][1] = new_pos[1];
      hotaru_pos[i][2] = new_pos[2];
    }
    else {
      reposit_hotaru_i(i);
      continue;
    }
  }
}


void
Hotaru::draw()
{
  if (!on) return;             // Now it's off.

  glPushAttrib(GL_LIGHTING_BIT | GL_POINT);
  glDisable(GL_LIGHTING); 
  glPointSize(2.0);
  glBegin(GL_POINTS);
  glColor3f(1.0, 1.0, 0.0);
  for (int i=0; i<HOTARU_NUM; i++) {
    glVertex3fv(hotaru_pos[i]);
  }
  glEnd();
  glPopAttrib();
}


void
Hotaru::button3_has_pressed()
{
  turn(0);
}


