
// Please read the file "LICENCE_ja.txt"(in Japanese) or "LICENCE_en.txt" (in English)
// before you use this software. 

#include "vfive.h"

extern Data*          data;
extern Tracer*        tracer;
extern LocalArrows*   localArrows;
extern LocalSlicer*   localSlicer;
extern Snow*          snow;

/*------------------------------------------------------------------
 > mesh_int_pos
 *------------------------------------------------------------------
 *
 *
 */

static void mesh_int_pos(ffloat_ x, ffloat_ y, ffloat_ z,
                         int *iout, int *jout, int *kout) {
  int i,j,k;
  int n1,n2,n3;

  n1 = data->n1;
  n2 = data->n2;
  n3 = data->n3;

  if (x < data->x[0] || x > data->x[n1-1] ||
      y < data->y[0] || y > data->y[n2-1] ||
      z < data->z[0] || z > data->z[n3-1] ) {
    puts("*** error: out of the box (mesh_int_pos)\n");
    exit(9);
  }

  for (i=n1-1; i>=0; i--) {
    if( data->x[i] <= x ) {
      *iout = i;
      break;
    }
  }
  
  for (j=n2-1; j>=0; j--) {
    if( data->y[j] <= y ) {
      *jout = j;
      break;
    }
  }

  for (k=n3-1; k>=0; k--) {
    if( data->z[k] <= z ) {
      *kout = k;
      break;
    }
  }
}

/*------------------------------------------------------------------
 > find_fact
 *------------------------------------------------------------------
 *
 *
 */

int find_fact(ffloat_ x, ffloat_ y, ffloat_ z,
	      int *i, int *j, int *k,
	      ffloat_ cx[4], ffloat_ cy[4], ffloat_ cz[4]) 
{
  int n1 = data->n1;
  int n2 = data->n2;
  int n3 = data->n3;
    
//  if ( x < data->region_xmin || x > data->region_xmax
//    || y < data->region_ymin || y > data->region_ymax
//    || z < data->region_zmin || z > data->region_zmax )  return(1);

  if ( x < data->region_xmin || x >= data->region_xmax
    || y < data->region_ymin || y >= data->region_ymax
    || z < data->region_zmin || z >= data->region_zmax )  return(1);

  mesh_int_pos(x, y, z, i, j, k);

  ffloat_ hx = ( x - data->x[*i] ) / (data->x[*i+1] - data->x[*i]);
  ffloat_ hy = ( y - data->y[*j] ) / (data->y[*j+1] - data->y[*j]);
  ffloat_ hz = ( z - data->z[*k] ) / (data->z[*k+1] - data->z[*k]);

  /* debug s */
  if( hx<0.0 || hx > 1.0 ) printf("  ** error.  hx = %f\n", hx);
  if( hy<0.0 || hy > 1.0 ) printf("  ** error.  hy = %f\n", hy);
  if( hz<0.0 || hz > 1.0 ) printf("  ** error.  hz = %f\n", hz);
  /* debug e */

  if (*i==0 || *i==n1-2) {
    cx[0] = (1.0-hx)/2.0;
    cx[1] = (1.0-hx)/2.0;
    cx[2] =       hx/2.0;
    cx[3] =       hx/2.0;
  }
  else {
    cx[0] =     -hx *(2.0-hx)*(1.0-hx)/6.0;
    cx[1] = (1.0+hx)*(2.0-hx)*(1.0-hx)/2.0;
    cx[2] =      hx *(2.0-hx)*(1.0+hx)/2.0;
    cx[3] =     -hx *(1.0-hx)*(1.0+hx)/6.0;
  }

  if (*j==0 || *j==n2-2) {
    cy[0] = (1.0-hy)/2.0;
    cy[1] = (1.0-hy)/2.0;
    cy[2] =       hy/2.0;
    cy[3] =       hy/2.0;
  }
  else {
    cy[0] =     -hy *(2.0-hy)*(1.0-hy)/6.0;
    cy[1] = (1.0+hy)*(2.0-hy)*(1.0-hy)/2.0;
    cy[2] =      hy *(2.0-hy)*(1.0+hy)/2.0;
    cy[3] =     -hy *(1.0-hy)*(1.0+hy)/6.0;
  }

  if (*k==0 || *k==n3-2) {
    cz[0] = (1.0-hz)/2.0;
    cz[1] = (1.0-hz)/2.0;
    cz[2] =       hz/2.0;
    cz[3] =       hz/2.0;
  }
  else {
    cz[0] =     -hz *(2.0-hz)*(1.0-hz)/6.0;
    cz[1] = (1.0+hz)*(2.0-hz)*(1.0-hz)/2.0;
    cz[2] =      hz *(2.0-hz)*(1.0+hz)/2.0;
    cz[3] =     -hz *(1.0-hz)*(1.0+hz)/6.0;
  }

  return(0); 
}


/*
 *------------------------------------------------------------------
 > interpolator
 *------------------------------------------------------------------
 *
 *
 */

ffloat_ interpolator(int i, int j, int k,
		    ffloat_ *cx, ffloat_ *cy, ffloat_ *cz,   // weight
		    ffloat_ ***q)     // 3D field to be interpolated
{
    ffloat_ b01, b02, b03, b04, b05, b06, b07, b08, b09, b10;
    ffloat_ b11, b12, b13, b14, b15, b16, b17, b18, b19, b20;
    int ip1,jp1,kp1,im1,jm1,km1,
        ip2,jp2,kp2;
    int n1,n2,n3;

    n1 = data->n1;
    n2 = data->n2;
    n3 = data->n3;

    im1 = i-1;
    jm1 = j-1;
    km1 = k-1; 
    ip1 = i+1; 
    ip2 = i+2;
    jp1 = j+1; 
    jp2 = j+2;
    kp1 = k+1; 
    kp2 = k+2;

    if(i==0) {
      im1 = i; 
      ip2 = ip1;
    }

    if(j==0) {
      jm1 = j;
      jp2 = jp1;
    }

    if(k==0) {
      km1 = k;
      kp2 = kp1;
    }

    if(i==n1-2) {
      ip2 = ip1;
      im1 = i;
    }

    if(j==n2-2) {
      jp2 = jp1;
      jm1 = j;
    }

    if(k==n3-2) {
      kp2 = kp1;
      km1 = k;
    }

    b01 = cx[0]*q[km1][jm1][im1] + cx[1]*q[km1][jm1][i  ]
        + cx[2]*q[km1][jm1][ip1] + cx[3]*q[km1][jm1][ip2];
    b02 = cx[0]*q[km1][j  ][im1] + cx[1]*q[km1][j  ][i  ]
        + cx[2]*q[km1][j  ][ip1] + cx[3]*q[km1][j  ][ip2];
    b03 = cx[0]*q[km1][jp1][im1] + cx[1]*q[km1][jp1][i  ]
        + cx[2]*q[km1][jp1][ip1] + cx[3]*q[km1][jp1][ip2];
    b04 = cx[0]*q[km1][jp2][im1] + cx[1]*q[km1][jp2][i  ]
        + cx[2]*q[km1][jp2][ip1] + cx[3]*q[km1][jp2][ip2];

    b05 = cx[0]*q[k  ][jm1][im1] + cx[1]*q[k  ][jm1][i  ]
        + cx[2]*q[k  ][jm1][ip1] + cx[3]*q[k  ][jm1][ip2];
    b06 = cx[0]*q[k  ][j  ][im1] + cx[1]*q[k  ][j  ][i  ]
        + cx[2]*q[k  ][j  ][ip1] + cx[3]*q[k  ][j  ][ip2];
    b07 = cx[0]*q[k  ][jp1][im1] + cx[1]*q[k  ][jp1][i  ]
        + cx[2]*q[k  ][jp1][ip1] + cx[3]*q[k  ][jp1][ip2];
    b08 = cx[0]*q[k  ][jp2][im1] + cx[1]*q[k  ][jp2][i  ]
        + cx[2]*q[k  ][jp2][ip1] + cx[3]*q[k  ][jp2][ip2];

    b09 = cx[0]*q[kp1][jm1][im1] + cx[1]*q[kp1][jm1][i  ]
        + cx[2]*q[kp1][jm1][ip1] + cx[3]*q[kp1][jm1][ip2];
    b10 = cx[0]*q[kp1][j  ][im1] + cx[1]*q[kp1][j  ][i  ]
        + cx[2]*q[kp1][j  ][ip1] + cx[3]*q[kp1][j  ][ip2];
    b11 = cx[0]*q[kp1][jp1][im1] + cx[1]*q[kp1][jp1][i  ]
        + cx[2]*q[kp1][jp1][ip1] + cx[3]*q[kp1][jp1][ip2];
    b12 = cx[0]*q[kp1][jp2][im1] + cx[1]*q[kp1][jp2][i  ]
        + cx[2]*q[kp1][jp2][ip1] + cx[3]*q[kp1][jp2][ip2];

    b13 = cx[0]*q[kp2][jm1][im1] + cx[1]*q[kp2][jm1][i  ]
        + cx[2]*q[kp2][jm1][ip1] + cx[3]*q[kp2][jm1][ip2];
    b14 = cx[0]*q[kp2][j  ][im1] + cx[1]*q[kp2][j  ][i  ]
        + cx[2]*q[kp2][j  ][ip1] + cx[3]*q[kp2][j  ][ip2];
    b15 = cx[0]*q[kp2][jp1][im1] + cx[1]*q[kp2][jp1][i  ]
        + cx[2]*q[kp2][jp1][ip1] + cx[3]*q[kp2][jp1][ip2];
    b16 = cx[0]*q[kp2][jp2][im1] + cx[1]*q[kp2][jp2][i  ]
        + cx[2]*q[kp2][jp2][ip1] + cx[3]*q[kp2][jp2][ip2];

    b17 = cy[0]*b01 + cy[1]*b02 + cy[2]*b03 + cy[3]*b04;
    b18 = cy[0]*b05 + cy[1]*b06 + cy[2]*b07 + cy[3]*b08;
    b19 = cy[0]*b09 + cy[1]*b10 + cy[2]*b11 + cy[3]*b12;
    b20 = cy[0]*b13 + cy[1]*b14 + cy[2]*b15 + cy[3]*b16;

    return(cz[0]*b17+cz[1]*b18+cz[2]*b19+cz[3]*b20);

    }

/*
 *------------------------------------------------------------------
 > intpol_get_scalar
 *------------------------------------------------------------------
 *
 *
 */

int 
intpol_get_scalar(int nth, ffloat_ *pos, ffloat_ *val)  
{

    ffloat_  cx[4], cy[4], cz[4];
    int     i,j,k,iout;

    iout = find_fact(pos[0], pos[1], pos[2],
                     &i, &j, &k, cx, cy, cz);

    if (iout) return(iout);

    *val = interpolator(i,j,k,cx,cy,cz,data->scalar_fld[nth]->val);


    return(iout);
}


/*
 *------------------------------------------------------------------
 > intpol_get_scalar_for_local_slicer
 *------------------------------------------------------------------
 *
 *
 */

int 
intpol_get_scalar_for_local_slicer(int nv, float *pos, ffloat_ *val)  
{
  ffloat_  cx[4], cy[4], cz[4];
  int     i,j,k;

  int iout = find_fact((ffloat_)pos[0], (ffloat_)pos[1], (ffloat_)pos[2], 
		       &i, &j, &k, cx, cy, cz);

  if (iout) return(iout);

  *val = interpolator(i,j,k,cx,cy,cz, data->scalar_fld[nv]->val);

  return(iout);
}

/*
 *------------------------------------------------------------------
 > intpol_get_vector
 *------------------------------------------------------------------
 *
 *
 */

int 
intpol_get_vector(int nv, ffloat_ pos[], ffloat_ vec[], ffloat_ *val) 
{
  ffloat_  cx[4], cy[4], cz[4];
  int i, j, k;

  int iout = find_fact(pos[0], pos[1], pos[2], 
		   &i, &j, &k, cx, cy, cz);

  if (iout) return(iout);

  vec[0] = interpolator(i,j,k,cx,cy,cz,data->vector_fld[nv]->x);
  vec[1] = interpolator(i,j,k,cx,cy,cz,data->vector_fld[nv]->y);
  vec[2] = interpolator(i,j,k,cx,cy,cz,data->vector_fld[nv]->z);

  *val = sqrt(vec[0]*vec[0]+vec[1]*vec[1]+vec[2]*vec[2]);

  return(iout);
}


/*
 *------------------------------------------------------------------
 > intpol_get_vect_for_tracer
 *------------------------------------------------------------------
 *
 *
 */

int 
intpol_get_vect_for_tracer(ffloat_ dummy, ffloat_ pos[], ffloat_ vec[], int n)
{
  ffloat_  cx[4], cy[4], cz[4];
  int i, j, k;

  int iout = find_fact(pos[0], pos[1], pos[2], 
		       &i, &j, &k, cx, cy, cz);

  if (iout) return(iout);

//  int n = tracer->svec();
  
  vec[0] = interpolator(i,j,k,cx,cy,cz,data->vector_fld[n]->x);
  vec[1] = interpolator(i,j,k,cx,cy,cz,data->vector_fld[n]->y);
  vec[2] = interpolator(i,j,k,cx,cy,cz,data->vector_fld[n]->z);

  return(iout);
}

/*
 *------------------------------------------------------------------
 > intpol_get_vect_for_lineAdvector
 *------------------------------------------------------------------
 *
 *
 */
/*
int 
intpol_get_vect_for_lineAdvector(ffloat_ dummy, ffloat_ pos[], ffloat_ vec[], int n)
{
  ffloat_  cx[4], cy[4], cz[4];
  int i, j, k;

  int iout = find_fact(pos[0], pos[1], pos[2], 
		       &i, &j, &k, cx, cy, cz);

  if (iout) return(iout);

//  int n = lineAdvector->svec();
  
  vec[0] = interpolator(i,j,k,cx,cy,cz,data->vector_fld[n]->x);
  vec[1] = interpolator(i,j,k,cx,cy,cz,data->vector_fld[n]->y);
  vec[2] = interpolator(i,j,k,cx,cy,cz,data->vector_fld[n]->z);

  return(iout);
}
*/
/*
 *------------------------------------------------------------------
 > intpol_get_normed_vect_for_tracer
 *------------------------------------------------------------------
 *
 *
 */

int 
intpol_get_normed_vect_for_tracer(ffloat_ dummy, ffloat_ pos[], ffloat_ v[], int n)
{
  int iout = intpol_get_vect_for_tracer(dummy, pos, v, n);
  if (iout) return(iout);

  ffloat_ vv = sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2]);

  if(vv < 1.0E-14){
	  v[0] = v[1] = v[2] = 0.0;
	  iout = 1;
	  return iout;}

  v[0] /= vv;
  v[1] /= vv;
  v[2] /= vv;

  return(iout);
}


/*
 *------------------------------------------------------------------
 > intpol_get_vect_for_local_arrow
 *------------------------------------------------------------------
 *
 *
 */

int 
intpol_get_vect_for_local_arrow(ffloat_ pos[], ffloat_ vec[], ffloat_ *val)
{
  ffloat_  cx[4], cy[4], cz[4];
  int i, j, k;

  int iout = find_fact(pos[0], pos[1], pos[2], 
		       &i, &j, &k, cx, cy, cz);

  if (iout) return(iout);

  int n = localArrows->svec();
  
  vec[0] = interpolator(i,j,k,cx,cy,cz,data->vector_fld[n]->x);
  vec[1] = interpolator(i,j,k,cx,cy,cz,data->vector_fld[n]->y);
  vec[2] = interpolator(i,j,k,cx,cy,cz,data->vector_fld[n]->z);

  *val = sqrt(vec[0]*vec[0]+vec[1]*vec[1]+vec[2]*vec[2]);

  return(iout);
}

/*
 *------------------------------------------------------------------
 > intpol_get_vect_for_local_slicer
 *------------------------------------------------------------------
 *
 *
 */

int 
intpol_get_vect_for_local_slicer(float pos[], float vec[], ffloat_ *val)
{
  ffloat_  cx[4], cy[4], cz[4];
  ffloat_  pos_dp[3];
  int     i, j, k;

  vec_f2ff(3, pos_dp, pos);

  int iout = find_fact(pos_dp[0], pos_dp[1], pos_dp[2], 
		       &i, &j, &k, cx, cy, cz);

  if (iout) return(iout);

  int n = localSlicer->svec();
  
  vec[0] = interpolator(i,j,k,cx,cy,cz,data->vector_fld[n]->x);
  vec[1] = interpolator(i,j,k,cx,cy,cz,data->vector_fld[n]->y);
  vec[2] = interpolator(i,j,k,cx,cy,cz,data->vector_fld[n]->z);

  *val = sqrt(vec[0]*vec[0]+vec[1]*vec[1]+vec[2]*vec[2]);

  return(iout);
}

/*
 *------------------------------------------------------------------
 > intpol_get_vect_for_snow
 *------------------------------------------------------------------
 *
 *
 */

int 
intpol_get_vect_for_snow(float pos[], float vec[], float* val)
{
  ffloat_  cx[4], cy[4], cz[4];
  ffloat_  pos_dp[3];
  int     i, j, k;

  vec_f2ff(3, pos_dp, pos);

  int iout = find_fact(pos_dp[0], pos_dp[1], pos_dp[2], 
		       &i, &j, &k, cx, cy, cz);

  if (iout) return(iout);

  int n = snow->svec();
  
  vec[0] = interpolator(i,j,k,cx,cy,cz,data->vector_fld[n]->x);
  vec[1] = interpolator(i,j,k,cx,cy,cz,data->vector_fld[n]->y);
  vec[2] = interpolator(i,j,k,cx,cy,cz,data->vector_fld[n]->z);

  *val = sqrt(vec[0]*vec[0]+vec[1]*vec[1]+vec[2]*vec[2]);

  return(iout);
}
