//
// vfive: Vector Field Interactive Visualization Environment
//
// local_arrow.c++
//
//                by Kageyama, Akira  (kage@nifs.ac.jp)  2000.03.18
//  
// Please read the file "LICENCE_ja.txt"(in Japanese) or "LICENCE_en.txt" (in English)
// before you use this software. 

#include "vfive.h"

extern Data *data;
extern GLUquadricObj *cyl[CAVE_MAX_WALLS];                   // cylinder

LocalArrows:: LocalArrows()
{
  on = 0;
  svec_ = 0;
  position_set = 0;

  beam = new PointerBeam(0.0, 0.3, 1.0, 3.0);
  if(beam == NULL) v5_error("can't allocate beam: local arrows");

  // from init
  int ndiv = 4;    
  ffloat_ radius = 2.0;               // radius, feet 
  ffloat_ ds = radius / ndiv;

  ffloat_ x,y,z,r;
  int ctr = 0;
  for(int i=-ndiv; i<ndiv; i++) {
    for(int j=-ndiv; j<ndiv; j++) {
      for(int k=-ndiv; k<ndiv; k++) {
	x = i*ds;
	y = j*ds;
	z = k*ds;
	r = sqrt(x*x+y*y+z*z);
	if ( r<= radius ) {
	  dx[ctr] = x;
	  dy[ctr] = y;
	  dz[ctr] = z;
	  ctr++;
	  if (ctr>= LOC_ARROWS_POINT_MAX) {
	    puts("   too many points in andon radius\n");
	    break;
	  }
	}
      }
    }
  }
  npoint = ctr;
  printf("v5:  local_arrows npoint = %d\n", npoint);
}

void
LocalArrows:: material()
{
    int tID;
    GLfloat diffuse0[]  = {0.3, 0.3, 1.0, 1.0};
    GLfloat ambient0[]  = {0.0, 0.3, 0.3, 1.0};
    GLfloat specular0[] = {0.3, 1.0, 0.3, 1.0};
    GLfloat diffuse1[]  = {0.8, 0.8, 0.3, 1.0};
    GLfloat ambient1[]  = {0.3, 0.3, 0.0, 1.0};
    GLfloat specular1[] = {0.6, 0.6, 0.6, 1.0};

    tID = CAVEUniqueIndex();
    mat0_dislis[tID] = glGenLists(1);
    glNewList(mat0_dislis[tID], GL_COMPILE);
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE,  diffuse0);
    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT,  ambient0);
    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular0);
    glMaterialf (GL_FRONT_AND_BACK, GL_SHININESS, 10.0);
    glEndList();

    mat1_dislis[tID] = glGenLists(1);
    glNewList(mat1_dislis[tID], GL_COMPILE);
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE,  diffuse1);
    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT,  ambient1);
    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular1);
    glMaterialf (GL_FRONT_AND_BACK, GL_SHININESS, 10.0);
    glEndList();
}

void 
LocalArrows:: initialize() 
{
  material();
}  


void
LocalArrows:: calc()
{
  if (!on) return;           // Now, it's off.

  ffloat_ pos[3], vec[3];
  float cylrad, stx, sty, stz, ux, uy, uz, r2, phi, tht;
  ffloat_ norm, vv, normh, nrmdv;
  int   nv, i;
  int tID = CAVEUniqueIndex();

  nv = svec();

  norm = 1.0/(data->vector_fld[nv]->vmax); /* 1.0 feet for max vector */
  normh = norm/2;

  cylrad = 0.02;

  glEnable(GL_LIGHTING);

  for(i=0; i<npoint; i++) {
    pos[0] = beam->tip[0] + dx[i];
    pos[1] = beam->tip[1] + dy[i];
    pos[2] = beam->tip[2] + dz[i];
    if (intpol_get_vect_for_local_arrow(pos, vec, &vv)) 
      continue; // out of box 
    stx = pos[0] - normh*vec[0];
    sty = pos[1] - normh*vec[1];
    stz = pos[2] - normh*vec[2];
    ux = norm*vec[0];
    uy = norm*vec[1];
    uz = norm*vec[2];
    nrmdv = norm*vv;
    r2 = sqrt(ux*ux+uy*uy);
    phi = inl_r2d(atan2(uy,ux));
    tht = inl_r2d(atan2(r2,uz));
    glPushMatrix();
      glTranslatef(stx, sty, stz);
      glRotatef(phi,0.0, 0.0, 1.0);
      glRotatef(tht,0.0, 1.0, 0.0);
      glCallList(mat0_dislis[tID]);
      gluCylinder(cyl[tID], cylrad, cylrad, nrmdv*0.75, 8, 2);
      glTranslatef(0.0, 0.0, nrmdv*0.75);
      glCallList(mat1_dislis[tID]);
      gluCylinder(cyl[tID], 2*cylrad, 0.0, nrmdv*0.25,8,2);
    glPopMatrix();
  }
}

void
LocalArrows:: draw()
{
  if (!on) return;

  glPushAttrib(GL_LIGHTING_BIT);
  if ( position_set ) {
    beam->draw();
  }
  calc();
  glPopAttrib();
}

void
LocalArrows:: button_has_pressed()
{
  beam->reset();//2008.4.21
  turn_on();
}
  

void
LocalArrows:: button_being_pressed()
{
  if(!on) return;// 2008.4.21
  position_set = 1;
  beam->reset();
}

void
LocalArrows:: button_has_released()
{
  position_set = 0;
}

void
LocalArrows:: button3_has_pressed()
{
  turn_off();
}



