//
// vfive: Vector Field Interactive Visualization Environment
//
//
// local_slicer.c++
//
//                by Kageyama, Akira  (kage@nifs.ac.jp)  2000.03.18
//  
// Please read the file "LICENCE_ja.txt"(in Japanese) or "LICENCE_en.txt" (in English)
// before you use this software. 



#include "vfive.h"

extern Data *data;

LocalSlicer:: LocalSlicer(int ndiv_, float side_)
{
  ndiv = ndiv_;
  side = side_;
  on = 0;
  sscal_ = 0;
  svec_ = 0;
  beam = new PointerBeam(1.0, 0.3, 1.0, 3.0);
  if(beam == NULL) v5_error("can't allocate beam: local slicer");
}


void 
LocalSlicer:: set_color(ffloat_ v, float *color, 
			ffloat_ wmax, ffloat_ wmin)  
{

  float hue, sat, val, r, g, b;
  float normed_v;


  normed_v = (wmax-v)/(wmax-wmin);


  hue = 0.7 * normed_v;
  sat = 1.0;
  val = 1.0;

  hsv2rgb(hue, sat, val, &r, &g, &b);

  color[0] = r;
  color[1] = g;
  color[2] = b;
}



void

LocalSlicer:: check_vmaxmin(float *a, float *dst, float *dsu,
			    ffloat_* vmax, ffloat_* vmin)
{
  float   posv[3];        // color normalization in the plane
  ffloat_  svval;          // color normalization in the plane

  *vmax = -1.e20;
  *vmin =  1.e20;

  int ss = sscal();

    *vmax = data->scalar_fld[ss]->vmax;
    *vmin = data->scalar_fld[ss]->vmin;

  return;

  for (int i=0; i<ndiv; i++) {
    for (int j=0; j<ndiv; j++) {
      posv[0] = a[0] + i*dst[0] + j*dsu[0];
      posv[1] = a[1] + i*dst[1] + j*dsu[1];
      posv[2] = a[2] + i*dst[2] + j*dsu[2];
      int iout = intpol_get_scalar_for_local_slicer(ss, posv, &svval);
      if (iout==0) {
	if (svval > *vmax ) *vmax = svval; 
	if (svval < *vmin ) *vmin = svval;
      }
    }
  }

}


void 
LocalSlicer:: slice(float *pos, float *norm)  
{
  float  sideh = side / 2;       // side length, feet 
  float  t[3], w[3], u[3], nrm;  // t & u are unit vectors
  float  dst[3], dsu[3], dst2u[3];
  ffloat_ sval;
  int iout, i;


  float v0[3], v1[3];       // vertex positions
  float c0[3], c1[3];        // colors
  float a[3];
  float black[] = {0.3, 0.3, 0.3};

  w[0] = 0.0, w[1] = 0.0, w[2] = 1.0;

  nrm = vec_cross_normalf(w,norm,t);

  if (nrm < 1.e-3) {                 // w and norm are parallel
    w[0] = 0.0, w[1] = 1.0, w[2] = 0.0;
    vec_cross_normalf(w,norm,t);
  }

  vec_cross_normalf(norm,t,u);
  /*
   *         u                  
   *         |
   *         |
   *         +---- t
   *
   */

  for (i=0; i<3; i++) {
    t[i] *= sideh;
    u[i] *= sideh;
    dst[i] = t[i] / ndiv; 
    dsu[i] = u[i] / ndiv; 
    dst2u[i] = dsu[i] - dst[i];
  }

  /*
   *       d             c
   *        +-----------+
   *        |           |
   *        |           |
   *        |           |
   *        +-----------+
   *       a             b
   */

  for (i=0; i<3; i++) {
    a[i] = pos[i] - t[i]/2 - u[i]/2;
  }


  ffloat_ wmax, wmin;

  check_vmaxmin(a, dst, dsu, &wmax, &wmin);

  glNormal3fv(norm);

  int ss = sscal();

  for (i=0; i<ndiv; i++) {
    glBegin(GL_TRIANGLE_STRIP);
      v0[0] = a[0] + i*dst[0];
      v0[1] = a[1] + i*dst[1];
      v0[2] = a[2] + i*dst[2];

      iout = intpol_get_scalar_for_local_slicer(ss, v0, &sval);
      if (iout==0) {
        set_color(sval, c0, wmax, wmin);
	glColor3fv(c0); 
      }
      else {
	glColor3fv(black); 
      }
      glVertex3fv(v0);

      for (int j=0; j<ndiv-1; j++) {
	/*                       */
	/*    \                  */
	/*     \                 */
	/*      \                */
	/*    ---  <-- This line */
	/*                       */

	v1[0] = v0[0] + dst[0];
	v1[1] = v0[1] + dst[1];
	v1[2] = v0[2] + dst[2];

	iout = intpol_get_scalar_for_local_slicer(ss, v1, &sval);

	if (iout==0) {
	  set_color(sval, c1, wmax, wmin);
	glColor3fv(c1); 
	}
	else {
	  glColor3fv(black); 
	}
	glVertex3fv(v1);

	/*                       */
	/*    \                  */
	/*     \ <-- This line.  */
	/*      \                */
	/*    ---                */
	/*                       */

	v0[0] = v1[0] + dst2u[0];
	v0[1] = v1[1] + dst2u[1];
	v0[2] = v1[2] + dst2u[2];
	iout = intpol_get_scalar_for_local_slicer(ss, v0, &sval);

	if (iout==0) {
	  set_color(sval, c0, wmax, wmin);
	  glColor3fv(c0); 
	}
	else {
	  glColor3fv(black); 
	}
	glVertex3fv(v0);
      }

      v1[0] = v0[0] + dst[0];
      v1[1] = v0[1] + dst[1];
      v1[2] = v0[2] + dst[2];
      iout = intpol_get_scalar_for_local_slicer(ss, v1, &sval);

      if (iout==0) {
        set_color(sval, c1, wmax, wmin);
	glColor3fv(c1); 
      }
      else {
	glColor3fv(black); 
      }
      glVertex3fv(v1);
    glEnd();
  }
}


void 
LocalSlicer:: calc()
{
  float  vec[3];
  ffloat_ vv;

  int iout = intpol_get_vect_for_local_slicer(beam->tip, vec, &vv);

  if (iout || fabs(vv)<1.e-5) {   // out of the region or zero vector
    slice(beam->tip, beam->direct);
  }
  else {
    vec_normalize(vec);
    slice(beam->tip, vec);
  }

}

void
LocalSlicer:: initialize()
{
  // Nothing.
}

void
LocalSlicer:: draw()
{
  if (!on) return;

  glPushAttrib(GL_LIGHTING_BIT);
  glDisable(GL_LIGHTING);
  beam->draw();
  calc();
  glPopAttrib();
}

void
LocalSlicer::button_being_pressed()
{
  if(!on) return; //2008.4.21
  beam->reset();
  beam->turn(1);
}

void
LocalSlicer::button_has_released()
{
  beam->turn(0);
}

void
LocalSlicer::button_has_pressed()
{
  beam->reset(); //2008.4.21
  beam->turn(1);
  turn_on();
}

void
LocalSlicer::button3_has_pressed()
{
  turn_off();
  beam->turn(0);
}

