//
// vfive: Vector Field Interactive Visualization Environment
//
//                by Kageyama, Akira  (kage@nifs.ac.jp)     2000.03.17
//                by Kageyama, Akira  (kage@jamstec.go.jp)  2003.12.18
//
// Volume Rendering (IRIX) and Scale Changer are added.
//                by Ohno, Nobuaki                          2006.6
//
// Some of codes are refined
//                by Ohno, Nobuaki                          2007.4
//
// Mutli-Thread version of VFIVE was developed              2008.4
//                by Ohno, Nobuaki
//
// main.c++
//  
// 
//          Akira Kageyama,
//          Theory and Computer Simulation Center.
//          National Institute for Fusion Science.
//          kage@nifs.ac.jp
// 
//
//
// Please read the file "LICENCE_ja.txt"(in Japanese) or "LICENCE_en.txt" (in English)
// before you use this software. 

#include "vfive.h"
Target * target;
Menu *menu;
Data *data;
Tracer *tracer;
LocalArrows *localArrows;
Isosurf *isosurf;
CurveObj *curveObj;
SurfaceObj *surfaceObj;
OrthoSlicer *orthoSlicer[3];	// xy, yz, and zx planes.
LocalSlicer *localSlicer;
BoundaryBox *boundaryBox;
Snow *snow;
Hotaru *hotaru;
Snapshot *snapshot;
cave_volrend *volumerendering;
scalefHandle *scalefh;
Probe *probe;
cave_message *cave_mssg;

char *inputFile;		// In vfive input data format.
char *curveObjFile = NULL;	// NULL means no object.
char *surfaceObjFile = NULL;

GLUquadricObj *sphereObj[CAVE_MAX_WALLS];	// Ball
GLUquadricObj *cyl[CAVE_MAX_WALLS];	// cylinder

int obj[2] = { 0, 0 };

int recalc_flag[CAVE_MAX_WALLS][5];	// 0:iso, 1:ortho_i, 2:ortho_j, 3:ortho_k, 4:volumerendering


void alloc_mem(void)
{

    data = new Data(inputFile);

    if (data == NULL)
	v5_error("can't allocate data: main");

    data->set();		// Read coord and field data from disk

    // Middle button 
    target = new Target;
    if (target == NULL)
	v5_error("can't allocate target: main");

    // Main menu panel
    menu = new Menu;
    if (menu == NULL)
	v5_error("can't allocate menu: main");

    // Tracer
    tracer = new Tracer;
    if (tracer == NULL)
	v5_error("can't allocate tracer: main");

    // LocalArrows
    localArrows = new LocalArrows;
    if (localArrows == NULL)
	v5_error("can't allocate local arrows: main");

    // Isosurface
    isosurf = new Isosurf;
    if (isosurf == NULL)
	v5_error("can't allocate isosurf: main");

    // CurveObj or "tubes"

    curveObj = new CurveObj;
    if (curveObj == NULL)
	v5_error("can't allocate curveObj: main");

    if (obj[0] != 0) {
	curveObj->make(curveObjFile);
	curveObj->read();
    }
    // SurfaceObj
    surfaceObj = new SurfaceObj;
    if (surfaceObj == NULL)
	v5_error("can't allocate surfaceObj: main");
    if (obj[1] != 0) {
	surfaceObj->make(surfaceObjFile);
	surfaceObj->read();
    }
    // Ortho slicer 
    orthoSlicer[0] = new OrthoSlicer(I, 0);
    if (orthoSlicer[0] == NULL)
	v5_error("can't allocate ortho[0]: main");

    orthoSlicer[1] = new OrthoSlicer(J, 0);
    if (orthoSlicer[1] == NULL)
	v5_error("can't allocate ortho[1]: main");

    orthoSlicer[2] = new OrthoSlicer(K, 0);
    if (orthoSlicer[2] == NULL)
	v5_error("can't allocate ortho[2]: main");


    // Local slicer
    localSlicer = new LocalSlicer(20, 5.0);
    if (localSlicer == NULL)
	v5_error("can't allocate local slicer: main");

    // Boundary box
    boundaryBox = new BoundaryBox;
    if (boundaryBox == NULL)
	v5_error("can't allocate boundarybox: main");

    // Snow
    snow = new Snow(10.0, 30.0);
    if (snow == NULL)
	v5_error("can't allocate snow: main");

    // Hotaru
    hotaru = new Hotaru;
    if (hotaru == NULL)
	v5_error("can't allocate hotaru: main");

    // Snapshot
    snapshot = new Snapshot;
    if (snapshot == NULL)
	v5_error("can't allocate snapshot: main");

    // scale handler
    scalefh = new scalefHandle;
    if (scalefh == NULL)
	v5_error("can't allocate scalefH: main");

    // Volume Rendering
    volumerendering = new cave_volrend;
    if (volumerendering == NULL)
	v5_error("can't allocate volume rendering: main");

    cave_mssg = new cave_message;
    if (cave_mssg == NULL)
	v5_error("can't allocate cave_message: main");

    probe = new Probe;
    if (probe == NULL)
	v5_error("can't allocate probe: main");

}

void init_gl()
{
    GLfloat light0_diffuse[] = { 1.0, 1.0, 1.0, 0.0 };
    GLfloat light0_ambient[] = { 0.3, 0.3, 0.3, 0.0 };
    GLfloat light0_specular[] = { 1.0, 1.0, 1.0, 0.0 };
    GLfloat light0_position[] = { 1.0, 1.0, 1.0, 0.0 };

    GLfloat light1_diffuse[] = { 1.0, 1.0, 1.0, 0.0 };
    GLfloat light1_ambient[] = { 0.3, 0.3, 0.3, 0.0 };
    GLfloat light1_specular[] = { 1.0, 1.0, 1.0, 0.0 };
    GLfloat light1_position[] = { -1.0, -1.0, -1.0, 0.0 };

    glShadeModel(GL_SMOOTH);
    glClearColor(0.0, 0.0, 0.0, 0.0);	// black background

    glEnable(GL_LIGHT0);	// Light 0
    glLightfv(GL_LIGHT0, GL_AMBIENT, light0_ambient);
    glLightfv(GL_LIGHT0, GL_SPECULAR, light0_specular);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, light0_diffuse);
    glLightfv(GL_LIGHT0, GL_POSITION, light0_position);

    glEnable(GL_LIGHT1);	// Light 1
    glLightfv(GL_LIGHT1, GL_AMBIENT, light1_ambient);
    glLightfv(GL_LIGHT1, GL_SPECULAR, light1_specular);
    glLightfv(GL_LIGHT1, GL_DIFFUSE, light1_diffuse);
    glLightfv(GL_LIGHT1, GL_POSITION, light1_position);

    glEnable(GL_LIGHTING);

    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);	// For textures
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);

    int tID = CAVEUniqueIndex();
    sphereObj[tID] = gluNewQuadric();
    cyl[tID] = gluNewQuadric();

    gluQuadricDrawStyle(cyl[tID], GLU_FILL);
    gluQuadricOrientation(cyl[tID], GLU_OUTSIDE);
    gluQuadricNormals(cyl[tID], GLU_SMOOTH);

    gluQuadricDrawStyle(sphereObj[tID], GLU_FILL);
    gluQuadricOrientation(sphereObj[tID], GLU_OUTSIDE);
    gluQuadricNormals(sphereObj[tID], GLU_SMOOTH);

    glColor3f(1.0, 1.0, 1.0);

}


void draw(void)
{
    float scale;
    scale = scalefh->current_scale_is();

    int tID = CAVEUniqueIndex();

    if (recalc_flag[tID][0] == 1) {
	recalc_iso();
	recalc_flag[tID][0] = 0;
    }

    if (recalc_flag[tID][1] == 1) {
	recalc_ortho_i();
	recalc_flag[tID][1] = 0;
    }

    if (recalc_flag[tID][2] == 1) {
	recalc_ortho_j();
	recalc_flag[tID][2] = 0;
    }

    if (recalc_flag[tID][3] == 1) {
	recalc_ortho_k();
	recalc_flag[tID][3] = 0;
    }

    if (recalc_flag[tID][4] == 1) {
	recalc_volrend();
	recalc_flag[tID][4] = 0;
    }

    glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);

    glPushMatrix();
    CAVENavTransform();		// CAVE navigation

    if (menu->on()) {		// show the menu if it's on. 

	menu->show();

    } else {

	if (volumerendering->ifon()) {
	    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	    glEnable(GL_BLEND);
	}

	glScalef(scale, scale, scale);

	boundaryBox->draw();

	tracer->draw();
	localArrows->draw();

	if (obj[0] != 0)
	    curveObj->draw();

	if (obj[1] != 0)
	    surfaceObj->draw();

	for (int i = 0; i < 3; i++) {
	    orthoSlicer[i]->draw();	// 3 ortho planes
	}

	localSlicer->draw();
	isosurf->draw();
	snow->draw();
	hotaru->draw();

	probe->draw();
	scalefh->draw();

	volumerendering->draw();

	if (volumerendering->ifon()) {
	    glDisable(GL_BLEND);
	}

    }

    glPopMatrix();

    snapshot->save_image();
}


void navigate(void)
{
    const float SPEED = 2.0;	// Old value = 10.0
    float jx = CAVE_JOYSTICK_X, jy = CAVE_JOYSTICK_Y, dt, t;
    static float prevtime = 0;

    //  When you would like to rotate the world, do it here.
    static int first = 1;
    if (first) {
	CAVENavTranslate(0.0, 0.0, -5.0);
	CAVENavRot(90.0, 'x');	// To make z direction upper.
	first = 0;
    }

    t = CAVEGetTime();		// standard techneque for the linear motion
    dt = t - prevtime;
    prevtime = t;

    if (fabs(jy) > 0.90) {
	float wandFront[3];

	CAVEGetVector(CAVE_WAND_FRONT, wandFront);
	CAVENavTranslate(wandFront[0] * jy * SPEED * dt,
			 wandFront[1] * jy * SPEED * dt,
			 wandFront[2] * jy * SPEED * dt);
    }

    if (fabs(jx) > 0.90)
	CAVENavRot(-jx * 30.0f * dt, 'y');

}


void parse_option(int argc, char **argv)
{
    // Getting main option (input field file name):  vfive input.v5
    if (argv[1] == NULL) {
	printf(" Usage: %s input.v5\n", argv[0]);
	exit(1);
    } else {
	inputFile = argv[1];
	for (int i = 2; i < argc; i++)
	    argv[i - 1] = argv[i];
	argc--;			// shift args 
	printf("v5: inputFile=%s\n", inputFile);
    }

    // Getting '-?' optoins
    for (int argi = 1; argi < argc && argv[argi][0] == '-'; argi++) {
	char p = argv[argi][1];
	switch (p) {
	case 'l':
	    argi++;
	    if (argi >= argc) {
		puts(" No file name for line object.");
		printf(" Usage: %s %s -l line_obj_file\n", argv[0],
		       inputFile);
		exit(1);
	    }
	    curveObjFile = argv[argi];
	    printf("v5: curveObjFile = %s\n", curveObjFile);
	    break;
	case 's':
	    argi++;
	    if (argi >= argc) {
		puts(" No file name for surface object.");
		printf(" Usage: %s %s -s surface_obj_file\n", argv[0],
		       inputFile);
		exit(1);
	    }
	    surfaceObjFile = argv[argi];
	    printf("v5: surfaceObjFile = %s\n", surfaceObjFile);
	    break;
	default:
	    puts(" Unknown option.");
	    exit(1);
	}
    }

}

void init()
{
    init_gl();			// OpenGL parameters

    menu->initialize();		// Menu

    tracer->initialize();	// Tracer
    localArrows->initialize();	// Local arrows
    isosurf->initialize();	// Isosurface

    if (obj[0] != 0)
	curveObj->initialize();	// Curve obj
    if (obj[1] != 0)
	surfaceObj->initialize();	// Surface obj

    localSlicer->initialize();	// Local slicer

    for (int i = 0; i < 3; i++) {
	orthoSlicer[i]->initialize();
    }

    boundaryBox->initialize();	// Boundary box
    hotaru->initialize();	// Hotaru
    volumerendering->initialize();	// volume rendering
}


void refresh()
{
    if (CAVEMasterDisplay()) {
	button1();
	button2();
	button3();
	probe->probe_calc();
	if (volumerendering->ifon()) {
	    volumerendering->plane_calc();	//Calculation of Vertex of Slices 
	}
    }

    CAVEDisplayBarrier();
}

int main(int argc, char **argv)
{
    for (int i = 0; i < CAVE_MAX_WALLS; i++) {
	recalc_flag[i][0] = 0;
	recalc_flag[i][1] = 0;
	recalc_flag[i][2] = 0;
	recalc_flag[i][3] = 0;
	recalc_flag[i][4] = 0;
    }

    parse_option(argc, argv);
    if (curveObjFile != NULL)
	obj[0] = 1;
    if (surfaceObjFile != NULL)
	obj[1] = 1;

    argc = 1;
    CAVEConfigure(&argc, argv, NULL);	// CAVE configuration set 

    alloc_mem();

    menu->make_panel_image();

    CAVEInit();
    CAVEInitApplication(init, 0);
    CAVEFrameFunction(refresh, 0);
    CAVEDisplay(draw, 0);

    while (!CAVEgetbutton(CAVE_ESCKEY) && menu->running) {
	navigate();

	if (CAVEDistribNumNodes() == 1) {
	    // single system
	    if (!menu->on()) {
		tracer->integrate();	// Done only when menu is off.
		snow->integrate();
		hotaru->integrate();
	    }

	} else {
	    // cluster system
	    tracer->integrate();
	    snow->integrate();
	    hotaru->integrate();
	}
	CAVEUSleep(1);

    }
//    menu->delete_panel_image();
//    CAVEExit();
    CAVEHalt();
    extern void delete_panel_image();
    delete_panel_image();

    return 0;
}
