//
// vfive: Vector Field Interactive Visualization Environment
//
//
// menu.c++
//
//                by Kageyama, Akira  (kage@nifs.ac.jp)  2000.03.17
//
//		  modified by N.Ohno 2008.1.31
//
// Please read the file "LICENCE_ja.txt"(in Japanese) or "LICENCE_en.txt" (in English)
// before you use this software. 

#include "vfive.h"
extern char*          inputFile;           // vfive input data.

extern Data*          data;
extern Target*        target;
extern Menu*          menu;
extern Tracer*        tracer;
extern LocalArrows*   localArrows;
extern Isosurf*       isosurf;
extern CurveObj*      curveObj;
extern SurfaceObj*    surfaceObj;
extern OrthoSlicer*   orthoSlicer[3];
extern LocalSlicer*   localSlicer;
extern BoundaryBox*   boundaryBox;
extern Snow*          snow;
extern Hotaru*        hotaru;
extern Snapshot*      snapshot;
extern cave_volrend*  volumerendering;
extern scalefHandle*  scalefh;
extern Probe*      probe;

//------------------------------------------------------------------------
// Panel name List
//
static Panel* pan_main;
static Panel* pan_vector_select;
static Panel* pan_scalar_select;
static Panel* pan_vector_vizMenu;
static Panel* pan_scalar_vizMenu;
static Panel* pan_orthoSlicer_planeSelect;
static Panel* pan_object_select;

extern int recalc_flag[CAVE_MAX_WALLS][5]; // 0:iso, 1:ortho_i, 2:ortho_j, 3:ortho_k 4:volume rendering
//
// Panel point must be in shared memory. Class menu has region
// for them ( panels[] ). Here we aliase them for easy reading.
//
void
Menu::name_the_panels()
{
  pan_main = panels[0];
  pan_vector_select = panels[1];
  pan_scalar_select = panels[2];
  pan_vector_vizMenu = panels[3];
  pan_scalar_vizMenu = panels[4];

  pan_orthoSlicer_planeSelect = panels[5];

  pan_object_select = panels[6];
}


//------------------------------------------------------------------------


static void 
F_vect_field_choice()
{
  menu->panel_at[1] = pan_vector_select;
  menu->set_depth(1);
}

static void
F_scal_field_choice()
{
//  menu->set_depth(1);
//  menu->panel_at[1] = pan_scalar_select; //2008.4.21
  menu->panel_at[1] = pan_scalar_select;
  menu->set_depth(1);
}

static void
F_vect_exec_kind_choice()
{
  menu->panel_at[2] = pan_vector_vizMenu;
  menu->set_depth(2);
}

static void
F_scal_exec_kind_choice()
{
  menu->panel_at[2] = pan_scalar_vizMenu;
  menu->set_depth(2);
}


static void
F_object()
{
  menu->panel_at[1] = pan_object_select;
  menu->set_depth(1);
}

static void
G_curve()
{
  curveObj->toggle_on_off();
}

static void
G_surface()
{
  puts("    surface is selected");
  surfaceObj->toggle_on_off();
}


static void
F_field_lines()
{
  menu->set_depth(2);  // Delete panels in deeer level (i.e. in the right).
}

static void
G_field_lines()
{
  target->set(TRACER);
  int nthvec = pan_vector_select->srow();  
  tracer->select_vec(nthvec);
  tracer->turn_on(BOTH);
  tracer->integration_on();
}

static void
F_part_tracer()
{
  menu->set_depth(2);  // Delete panels in deeer level (i.e. in the right).
}


static void
G_part_tracer()
{
  target->set(TRACER);
  int nthvec = pan_vector_select->srow();  
  tracer->select_vec(nthvec);
  tracer->turn_on(POSI);
  tracer->integration_on();
}

static void
F_arrows()
{
  menu->set_depth(2);  // Delete panels in deeer level (i.e. in the right).
}

static void
G_arrows()
{
  menu->set_depth(2);
  target->set(LOCAL_ARROWS);
  int nthvec = pan_vector_select->srow();  
  localArrows->select_vec(nthvec);
}

static void
F_snow()
{
  menu->set_depth(2);  // Delete panels in deeer level (i.e. in the right).
}

static void
G_snow()
{
  target->set(SNOW);
  int nthvec = pan_vector_select->srow();  
  snow->select_vec(nthvec);
  snow->initialize();
  snow->turn(1);
}

static void
F_hotaru()
{
  menu->set_depth(2);  // Delete panels in deeer level (i.e. in the right).
}

static void
G_hotaru()
{
  target->set(HOTARU);
  int nthvec = pan_vector_select->srow();  
  hotaru->select_vec(nthvec);
  hotaru->place_hotaru();
  hotaru->turn(1);
}

static void
F_isos()
{
  menu->set_depth(2);  // Delete panels in deeer level (i.e. in the right).
}

static void
G_isos()
{
  target->set(ISOSURF);
  int nthscal = pan_scalar_select->srow();  
  isosurf->select_scal(nthscal);
  isosurf->turn(1);
}

static void
F_volrend()
{
  menu->set_depth(2);  // Delete panels in deeer level (i.e. in the right).
}

static void
G_volrend()
{
  target->set(VOLUME_RENDER);
  int nthscal = pan_scalar_select->srow();  
  volumerendering->select_scal(nthscal);
  volumerendering->turn_on();
}


static void
F_vprobe()
{
  menu->set_depth(2);  // Delete panels in deeer level (i.e. in the right).
}

static void
G_vprobe()
{
  target->set(PROBE);
  int nvec = pan_vector_select->srow();  
  probe->vec_turn_on(nvec);
}

static void
F_sprobe()
{
  menu->set_depth(2);  // Delete panels in deeer level (i.e. in the right).
}

static void
G_sprobe()
{
  target->set(PROBE);
  int nscal = pan_scalar_select->srow();  
  probe->scal_turn_on(nscal);
}


static void
F_local_slice()
{
  menu->set_depth(2);  // Delete panels in deeer level (i.e. in the right).
}

static void
G_local_slice()
{
  int nthscal = pan_scalar_select->srow();  
  localSlicer->select_scal(nthscal);
  target->set(LOCAL_SLICER);
}


static void
F_ortho_slice()
{
  menu->panel_at[3] = pan_orthoSlicer_planeSelect;
  menu->set_depth(3);
}


static void
G_ortho_slice_i()
{
  int nthscal = pan_scalar_select->srow();  
  orthoSlicer[0]->select_scal(nthscal);
  target->set(ORTHO_SLICER_I);
    for(int i=0;i<CAVENumPipes();i++)
	recalc_flag[i][1] = 1;
  orthoSlicer[0]->turn_on();
}

static void
G_ortho_slice_j()
{
  int nthscal = pan_scalar_select->srow();  
  orthoSlicer[1]->select_scal(nthscal);
  target->set(ORTHO_SLICER_J);
    for(int i=0;i<CAVENumPipes();i++)
	recalc_flag[i][2] = 1;
  orthoSlicer[1]->turn_on();
}

static void
G_ortho_slice_k()
{
  int nthscal = pan_scalar_select->srow();  
  orthoSlicer[2]->select_scal(nthscal);
  target->set(ORTHO_SLICER_K);
    for(int i=0;i<CAVENumPipes();i++)
	recalc_flag[i][3] = 1;
  orthoSlicer[2]->turn_on();
}


static void
F_snapshot()
{
  menu->set_depth(0);
}


static void
G_snapshot()
{
  target->set(SNAPSHOT);
  puts(" Snapshot is active....");
}

static void
F_scalef()
{
  menu->set_depth(0);
}


static void
G_scalef()
{
  target->set(SCALE_HANDLE);
}


static void
F_quit()
{
  menu->set_depth(0);
}

static void
G_quit()
{
  menu->running = 0;
  // CAVEExit(); 2008.4.8
}

//------------------------------------------------------------------------
static char file_name0[6][20] ={
 "Vector",
 "Scalar",   
 "Object",
 "Scalef",
 "Snapshot",
 "Quit" 
};

static char file_name1[MAX_NVEC][20];
static char file_name2[MAX_NSCAL][20];

static char file_name3[6][20] ={
 "Field_lines",
 "Particle_tracer",
 "Arrows",
 "Snow",
 "Hotaru",
 "ProbeV"
};

static char file_name4[5][20] ={
 "Isosurface",
 "Local_Slicer",
 "Ortho_Slicer",
 "Volume_Rendering",
 "ProbeS"
};

static char file_name5[3][20] ={
 "xy-plane",
 "yz-plane",
 "zx-plane"
};

static char file_name6[2][20] ={
 "curve",
 "surface"
};

void
Menu::make_panel_image()
{
  // When you add a new panel, increase "panel_num".

  int i;

  panel_num = 7;
  if (panel_num >= PANEL_NUM_MAX) v5_error(" Increase PANEL_NUM_MAX.");

  //-----------------------------
  // Make texture images on disk.
  //-----------------------------

  pan_main->make_image("Vector\\nSelect", 0x000000, 0xabcdef, file_name0[0]);   
  pan_main->make_image("Scalar\\nSelect", 0x000000, 0xabcdef, file_name0[1]);
  pan_main->make_image("Object\\nSelect", 0x000000, 0xabcdef, file_name0[2]);
  pan_main->make_image("Scale\\nChanger", 0x000000, 0xabcdef, file_name0[3]);
  pan_main->make_image("Snapshot", 0x000000, 0xabcdef, file_name0[4]);
  pan_main->make_image("Quit", 0xff0000, 0xabcdef, file_name0[5]);


  for (i=0; i<data->nvec; i++) {
    char key[20], label[50];

      sprintf(key, "vect%d_label", i);   
      v5_get_string(inputFile, key, label);  

    sprintf(file_name1[i], "vectfile%d", i);
    pan_vector_select->make_image(label,0x0000ff,0x88ff88, file_name1[i]);
  }

  for (i=0; i<data->nscal; i++) {
    char key[20], label[50];
    sprintf(key, "scal%d_label", i);   
    v5_get_string(inputFile, key, label);  

    sprintf(file_name2[i], "scalfile%d", i);
    pan_scalar_select->make_image(label,0x0000ff,0x88ff88, file_name2[i]);
  }

  pan_vector_vizMenu->make_image("Field\\nLines", 0x000000, 0xffffff, file_name3[0]);
  pan_vector_vizMenu->make_image("Particle\\nTracer", 0x000000, 0xffffff, file_name3[1]);
  pan_vector_vizMenu->make_image("Local\\nArrows", 0x000000, 0xffffff, file_name3[2]);
  pan_vector_vizMenu->make_image("Snow", 0x000000, 0xffffff, file_name3[3]);
  pan_vector_vizMenu->make_image("Hotaru", 0x000000, 0xffffff, file_name3[4]);
  pan_vector_vizMenu->make_image("Probe", 0x000000, 0xffffff, file_name3[5]);


  pan_scalar_vizMenu->make_image("Isosurface", 0x000000, 0xabefcd, file_name4[0]);
  pan_scalar_vizMenu->make_image("Local\\nSlicer", 0x000000, 0xabefcd, file_name4[1]);
  pan_scalar_vizMenu->make_image("Ortho\\nSlicer", 0x000000, 0xabefcd, file_name4[2]);
  pan_scalar_vizMenu->make_image("Volume\\nRendering", 0x000000, 0xabefcd, file_name4[3]);
  pan_scalar_vizMenu->make_image("Probe", 0x000000, 0xabefcd, file_name4[4]);

  pan_orthoSlicer_planeSelect->make_image("YZ-plane", 0x0000ff, 0xaaffaa, file_name5[0]);
  pan_orthoSlicer_planeSelect->make_image("ZX-plane", 0x0000ff, 0xaaffaa, file_name5[1]);
  pan_orthoSlicer_planeSelect->make_image("XY-plane", 0x0000ff, 0xaaffaa, file_name5[2]);

  pan_object_select->make_image("lines", 0x000000, 0x00ffff, file_name6[0]);
  pan_object_select->make_image("surfaces", 0x000000, 0x00ffff, file_name6[1]);

}


void
Menu::set_panels()
{
 int i;
  //------------------------------------
  // Set texture images on the frame.
  //------------------------------------
 for(i=0;i<6;i++){
  pan_main->set_image(i, file_name0[i]);   
 }

  for (i=0; i<data->nvec; i++) {
    pan_vector_select->set_image(i,file_name1[i]);
  }

  for (i=0; i<data->nscal; i++) {
    pan_scalar_select->set_image(i,file_name2[i]);
  }

 for(i=0;i<6;i++){
  pan_vector_vizMenu->set_image(i, file_name3[i]);
 }

 for(i=0;i<5;i++){
  pan_scalar_vizMenu->set_image(i, file_name4[i]);
 }

 for(i=0;i<3;i++){
  pan_orthoSlicer_planeSelect->set_image(i, file_name5[i]);
 }

 for(i=0;i<2;i++){
  pan_object_select->set_image(i, file_name6[i]);
 }

  // display list
  pan_main->make_display_list();   

  pan_vector_select->make_display_list();
  pan_scalar_select->make_display_list();

  pan_vector_vizMenu->make_display_list();
  pan_scalar_vizMenu->make_display_list();
  pan_orthoSlicer_planeSelect->make_display_list();
  pan_object_select->make_display_list();
}

void
Menu::delete_panel_image(void){
  //------------------------------------
  // Delete texture images on the Disk.
  //------------------------------------
 int i;
 char command[256];
#ifndef WIN32
 char del[] ="rm";
#else
 char del[] ="del";
#endif

 puts("deleting texture images");

 for(i=0;i<6;i++){
  sprintf(command, "%s %s", del, file_name0[i]);
  system(command);
 }

  for (i=0; i<data->nvec; i++) {
  sprintf(command, "%s %s", del, file_name1[i]);
  system(command);
  }

  for (i=0; i<data->nscal; i++) {
  sprintf(command, "%s %s", del, file_name2[i]);
  system(command);
  }

 for(i=0;i<6;i++){
  sprintf(command, "%s %s", del, file_name3[i]);
  system(command);
 }

 for(i=0;i<5;i++){
  sprintf(command, "%s %s", del, file_name4[i]);
  system(command);
 }

 for(i=0;i<3;i++){
  sprintf(command, "%s %s", del, file_name5[i]);
  system(command);
 }

 for(i=0;i<2;i++){
  sprintf(command, "%s %s", del, file_name6[i]);
  system(command);
 }

}

void delete_panel_image(void){
  //------------------------------------
  // Delete texture images on the Disk.
  //------------------------------------
 int i;
 char command[256];
#ifndef WIN32
 char del[] ="rm";
#else
 char del[] ="del";
#endif

 puts("deleting texture images");

 for(i=0;i<6;i++){
  sprintf(command, "%s %s", del, file_name0[i]);
  system(command);
 }

  for (i=0; i<data->nvec; i++) {
  sprintf(command, "%s %s", del, file_name1[i]);
  system(command);
  }

  for (i=0; i<data->nscal; i++) {
  sprintf(command, "%s %s", del, file_name2[i]);
  system(command);
  }

 for(i=0;i<6;i++){
  sprintf(command, "%s %s", del, file_name3[i]);
  system(command);
 }

 for(i=0;i<5;i++){
  sprintf(command, "%s %s", del, file_name4[i]);
  system(command);
 }

 for(i=0;i<3;i++){
  sprintf(command, "%s %s", del, file_name5[i]);
  system(command);
 }

 for(i=0;i<2;i++){
  sprintf(command, "%s %s", del, file_name6[i]);
  system(command);
 }

}

void
Menu::initialize()
{
  set_panels();
}

void
Menu::show()
{
  float scale;
  scale = scaleintp();

  glScalef(scale, scale, scale);
   beam->draw();
  glScalef(1.0/scale, 1.0/scale, 1.0/scale);

  for (int i=0; i <= depth; i++)
    panel_at[i]->show();
}


Menu::Menu()
{

  beam = new PointerBeam(1.0, 0.3, 0.3, 15.0);
   if(beam == NULL) v5_error("can't allocate beam: menu");

  for (int i=0; i<PANEL_NUM_MAX; i++){
    panels[i] = new Panel;
    if(panels[i] == NULL) v5_error("can't allocate panel: menu");
  }

  set_depth(-1);       // No menu

  name_the_panels();

  // these came from set()
  distance = 7.0;   // feet away.
  top_pos = 1.0;
  panel_w = 1.5;    // width (feet)
  panel_h = panel_w * float(PANEL_IMAGE_H) / float(PANEL_IMAGE_W);
  image_w = panel_w * 0.8;
  image_h = panel_h * 0.8;
  dphi = panel_w/distance; // radian
  shoot_ball_rad = panel_h*0.4;
  //
  running = 1; //2008.4.8

  // When you add a new panel, increase "panel_num".
  panel_num = 7;
  if (panel_num >= PANEL_NUM_MAX) v5_error(" Increase PANEL_NUM_MAX.");

  pan_main->make(0,
		 "Vector",   F_vect_field_choice, NULL,
		 "Scalar",   F_scal_field_choice, NULL,
		 "Object",   F_object,            NULL,
		 "Scalef",   F_scalef,            G_scalef,
		 "Snapshot", F_snapshot,          G_snapshot,
		 "Quit",     F_quit,              G_quit); 
  
  pan_vector_select->make2(1, data->nvec, "vect_", F_vect_exec_kind_choice);

  pan_scalar_select->make2(1, data->nscal, "scal_", F_scal_exec_kind_choice);


  pan_vector_vizMenu->make(2,
			   "Field_lines",     F_field_lines,  G_field_lines,
			   "Particle_tracer", F_part_tracer,  G_part_tracer,
			   "Arrows",          F_arrows,  G_arrows,
			   "Snow",            F_snow,  G_snow,
			   "Hotaru",          F_hotaru,  G_hotaru,
			   "Probe",           F_vprobe,  G_vprobe);

  pan_scalar_vizMenu->make(2,
			   "Isosurface",      F_isos,         G_isos,
			   "Local_Slicer",    F_local_slice,  G_local_slice,
			   "Ortho_Slicer",    F_ortho_slice,  NULL,
			   "volume-rendering",F_volrend,      G_volrend,
			   "Probe",F_sprobe,      G_sprobe);



  pan_orthoSlicer_planeSelect->make(3,
				    "xy-plane",  NULL,  G_ortho_slice_i,
				    "yz-plane",  NULL,  G_ortho_slice_j,
				    "zx-plane",  NULL,  G_ortho_slice_k);

  pan_object_select->make(1,
			  "curve",   NULL,  G_curve,
			  "surface", NULL,  G_surface);


}

void
Menu::button_being_pressed()
{
  int temp_depth = depth; //ohno 2008.4.4

  if(temp_depth < 0) return; //2008.4.21 for safty

  beam->reset();

  for (int i=0; i<=temp_depth; i++) {
    panel_at[i]->check_and_exec_f(beam);
  }
}

void
Menu::button_has_pressed()
{
//  set_depth(0);                           // Main menu on.
  panel_at[0] = pan_main;
  beam->reset();			//2008.4.21
  beam->turn(1);                       // turn on the beam.
  snapstate.record_head_info();
  appear_angle_phi = snapstate.head_phi()
                   + inl_d2r(10.0);    // slightly left.
  set_depth(0);                           // Main menu on.//2008.4.21
}

void
Menu::button_has_released()
{
  beam->turn(0);   // turn off the beam.
  for (int i=0; i<=depth; i++) {
    Panel* p = panel_at[i];
    p->check_and_exec_g();
  }
  set_depth(-1);      // Erase the menu.
}

