//
// vfive: Vector Field Interactive Visualization Environment
//
//
// ortho_slicer.c++
//
//                by Kageyama, Akira  (kage@nifs.ac.jp)  2000.03.18
//  
// Please read the file "LICENCE_ja.txt"(in Japanese) or "LICENCE_en.txt" (in English)
// before you use this software. 

#include "vfive.h"

extern Data* data;

OrthoSlicer:: OrthoSlicer(axis_ a, int i)
{
  on = 0;
  axis = a; 
  plane = i;
  handle = new OrthoSlicerHandle(axis, plane);
   if(handle == NULL) v5_error("can't allocate ortho handle : ortho");
  select_scal(0);    // default scalar field 0.
}

void
OrthoSlicer:: set_color(ffloat_ v, float *color)  
{
  float r, g, b;
  
  float normed_v;// = (vmax-v)/(vmax-vmin);
  normed_v = (vmax-v)/(vmax-vmin);

  float hue;// = 0.7 * normed_v;
  hue = 0.7 * normed_v;
  float sat = 1.0;
  float val = 1.0;

  hsv2rgb(hue, sat, val, &r, &g, &b);

  color[0] = r;
  color[1] = g;
  color[2] = b;

}

void
OrthoSlicer:: find_maxmin_i(int ss, int icut)
{
  vmax = -1.e20;
  vmin =  1.e20;

  vmax = data->scalar_fld[ss]->vmax;
  vmin = data->scalar_fld[ss]->vmin;

  return;

  for (int j=0; j<data->n2; j++) {
    for (int k=0; k<data->n3; k++) {
      ffloat_ val = data->scalar_fld[ss]->val[k][j][icut];
      if ( val > vmax ) vmax = val;
      if ( val < vmin ) vmin = val;
    }
  }
}

void
OrthoSlicer:: find_maxmin_j(int ss, int jcut)
{
  vmax = -1.e20;
  vmin =  1.e20;

  vmax = data->scalar_fld[ss]->vmax;
  vmin = data->scalar_fld[ss]->vmin;

  return;


  for (int k=0; k<data->n3; k++) {
    for (int i=0; i<data->n1; i++) {
      ffloat_ val = data->scalar_fld[ss]->val[k][jcut][i];
      if ( val > vmax ) vmax = val;
      if ( val < vmin ) vmin = val;
    }
  }
}

void
OrthoSlicer:: find_maxmin_k(int ss, int kcut)
{
  vmax = -1.e20;
  vmin =  1.e20;

  vmax = data->scalar_fld[ss]->vmax;
  vmin = data->scalar_fld[ss]->vmin;

  return;

  for (int i=0; i<data->n1; i++) {
    for (int j=0; j<data->n2; j++) {
      ffloat_ val = data->scalar_fld[ss]->val[kcut][j][i];
      if ( val > vmax ) vmax = val;
      if ( val < vmin ) vmin = val;
    }
  }
}
  

void
OrthoSlicer:: ortho_slicer_i(int icut)
{
  float v0[3], v1[3], v2[3];               // vertex positions
  float c0[3], c1[3], c2[3];               // colors
  
  int n3 = data->n3;
  int n2 = data->n2;

  int    ss = sscal();
  float normal[] = {1.0, 0.0, 0.0};        // x-axis.

  find_maxmin_i(ss, icut);

  glNormal3fv(normal);
  for (int j=0; j<n2-1; j++) {
    glBegin(GL_TRIANGLE_STRIP);
      v0[0] = data->x[icut];
      v0[1] = data->y[j];
      v0[2] = data->z[0];
      set_color(data->scalar_fld[ss]->val[0][j][icut], c0);
      glColor3fv(c0); 
      glVertex3fv(v0);
    for (int k=0; k<n3-1; k++) {
      v1[0] = data->x[icut];
      v1[1] = data->y[j+1]; 
      v1[2] = data->z[k];               
      set_color(data->scalar_fld[ss]->val[k][j+1][icut], c1);
      glColor3fv(c1); 
      glVertex3fv(v1);
      v2[0] = data->x[icut];
      v2[1] = data->y[j];
      v2[2] = data->z[k+1];               
      set_color(data->scalar_fld[ss]->val[k+1][j][icut], c2);
      glColor3fv(c2); 
      glVertex3fv(v2);
    }
      v0[0] = data->x[icut];
      v0[1] = data->y[j+1];
      v0[2] = data->z[n3-1];            
      set_color(data->scalar_fld[ss]->val[n3-1][j+1][icut], c0);
      glColor3fv(c0); 
      glVertex3fv(v0);
    glEnd();
  }
}

void
OrthoSlicer:: ortho_slicer_j(int jcut)
{
  float v0[3], v1[3], v2[3];               // vertex positions
  float c0[3], c1[3], c2[3];               // colors
  
  int n3 = data->n3;
  int n1 = data->n1;

  int    ss = sscal();
  float normal[] = {0.0, 1.0, 0.0};        // z-axis.

  find_maxmin_j(ss, jcut);

  glNormal3fv(normal);
  for (int k=0; k<n3-1; k++) {
    glBegin(GL_TRIANGLE_STRIP);
      v0[0] = data->x[0]; 
      v0[1] = data->y[jcut];
      v0[2] = data->z[k];
      set_color(data->scalar_fld[ss]->val[k][jcut][0], c0);
      glColor3fv(c0); 
      glVertex3fv(v0);
    for (int i=0; i<n1-1; i++) {
      v1[0] = data->x[i];
      v1[1] = data->y[jcut];
      v1[2] = data->z[k+1];      
      set_color(data->scalar_fld[ss]->val[k+1][jcut][i], c1);
      glColor3fv(c1); 
      glVertex3fv(v1);
      v2[0] = data->x[i+1]; 
      v2[1] = data->y[jcut];
      v2[2] = data->z[k];            
      set_color(data->scalar_fld[ss]->val[k][jcut][i+1], c2);
      glColor3fv(c2); 
      glVertex3fv(v2);
    }
      v0[0] = data->x[n1-1];
      v0[1] = data->y[jcut];
      v0[2] = data->z[k+1];              
      set_color(data->scalar_fld[ss]->val[k+1][jcut][n1-1], c0);
      glColor3fv(c0); 
      glVertex3fv(v0);
    glEnd();
  }
}

void
OrthoSlicer:: ortho_slicer_k(int kcut)
{
  float v0[3], v1[3], v2[3];               // vertex positions
  float c0[3], c1[3], c2[3];               // colors
  
  int n1 = data->n1;
  int n2 = data->n2;

  int    ss = sscal();
  float normal[] = {0.0, 0.0, 1.0};        // z-axis.

  find_maxmin_k(ss, kcut);

  glNormal3fv(normal);
  for (int i=0; i<n1-1; i++) {
    glBegin(GL_TRIANGLE_STRIP);
      v0[0] = data->x[i];
      v0[1] = data->y[0];
      v0[2] = data->z[kcut];
      set_color(data->scalar_fld[ss]->val[kcut][0][i], c0);
      glColor3fv(c0); 
      glVertex3fv(v0);
    for (int j=0; j<n2-1; j++) {
      v1[0] = data->x[i+1];
      v1[1] = data->y[j];
      v1[2] = data->z[kcut];
      set_color(data->scalar_fld[ss]->val[kcut][j][i+1], c1);
      glColor3fv(c1); 
      glVertex3fv(v1);
      v2[0] = data->x[i];
      v2[1] = data->y[j+1];
      v2[2] = data->z[kcut];
      set_color(data->scalar_fld[ss]->val[kcut][j+1][i], c2);
      glColor3fv(c2); 
      glVertex3fv(v2);
    }
      v0[0] = data->x[i+1];
      v0[1] = data->y[n2-1];
      v0[2] = data->z[kcut];
      set_color(data->scalar_fld[ss]->val[kcut][n2-1][i+1], c0);
      glColor3fv(c0); 
      glVertex3fv(v0);
    glEnd();
  }
}

void
OrthoSlicer:: calc()
{

  glNewList(dislis[CAVEUniqueIndex()], GL_COMPILE);
  glDisable(GL_LIGHTING);
    switch (axis) {
    case I:
      ortho_slicer_i(plane);
      break;
    case J:
      ortho_slicer_j(plane);
      break;
    case K:
      ortho_slicer_k(plane);
      break;
    }
  glEndList();
}


void
OrthoSlicer:: initialize()
{
  dislis[CAVEUniqueIndex()] = glGenLists(1);
  handle->initialize();
}

void
OrthoSlicer:: draw()
{
  float scale;

  scale = scaleintp();

  if (!on) return;

  glPushAttrib(GL_LIGHTING_BIT);
   glCallList(dislis[CAVEUniqueIndex()]);

   glScalef(1.0/scale,1.0/scale,1.0/scale);
    handle->draw();
   glScalef(scale,scale,scale);

  glPopAttrib();
}

  
void
OrthoSlicer:: button_has_pressed()
{
  handle->snap();
  handle->turn(1); //2008.4.21
}
  
void
OrthoSlicer:: button3_has_pressed()
{
  handle->turn(0);
  turn_off();
}
  
void
OrthoSlicer:: button_being_pressed()
{
//  handle->turn(1); //2008.4.21
  handle->wand_motion(plane);
}


void
OrthoSlicer:: button_has_released()
{
  plane = handle->cut_plane_is();
  handle->turn(0);
  turn_on();
}
