//
// vfive: Vector Field Interactive Visualization Environment
//
//
// ortho_slicer_handler.c++
//
//                by Kageyama, Akira  (kage@nifs.ac.jp)  2000.04.14
//  
// Please read the file "LICENCE_ja.txt"(in Japanese) or "LICENCE_en.txt" (in English)
// before you use this software. 

#include "vfive.h"

extern Data* data;
extern BoundaryBox* boundaryBox;

OrthoSlicerHandle:: OrthoSlicerHandle(axis_ a, int i)
{
  on = 0;
  axis = a; 
  cut_plane = i;

  float xl = data->region_xmax - data->region_xmin;
  float box_size_in_x = 1.0;             // feet
  scale = box_size_in_x / xl;            // in vfive.h
}

static void
blink(float t)
{
  Color col;

  if ( int(t*10) % 2 == 0 )      // blink every 0.1 second.
    glColor3fv(col.kiiro(0.0));
  else
    glColor3fv(col.aka(0.0));
}

void
OrthoSlicerHandle:: plane_xy(float t)
{
  float xmin = data->region_xmin;
  float xmax = data->region_xmax;
  float ymin = data->region_ymin;
  float ymax = data->region_ymax;
  float dx = (xmax - xmin) / 10;
  float dy = (ymax - ymin) / 10;
  float  z = data->z[cut_plane];

  glPushAttrib(GL_LINE_BIT);
  blink(t);
  glLineWidth(2);
  glBegin(GL_LINES);
    for (int i=0; i<=10; i++) {  
      float x = xmin + dx*i;
      glVertex3f(x,ymin,z);
      glVertex3f(x,ymax,z);
    }
    for (int j=0; j<=10; j++) {  
      float y = ymin + dy*j;
      glVertex3f(xmin,y,z);
      glVertex3f(xmax,y,z);
    }
  glEnd();
  glPopAttrib();
}

void
OrthoSlicerHandle:: plane_yz(float t)
{
  float ymin = data->region_ymin;
  float ymax = data->region_ymax;
  float zmin = data->region_zmin;
  float zmax = data->region_zmax;
  float dy = (ymax - ymin) / 10;
  float dz = (zmax - zmin) / 10;
  float  x = data->x[cut_plane];

  glPushAttrib(GL_LINE_BIT);

  blink(t);

  glLineWidth(2);
  glBegin(GL_LINES);
    for (int j=0; j<=10; j++) {  
      float y = ymin + dy*j;
      glVertex3f(x,y,zmin);
      glVertex3f(x,y,zmax);
    }
    for (int k=0; k<=10; k++) {  
      float z = zmin + dz*k;
      glVertex3f(x,ymin,z);
      glVertex3f(x,ymax,z);
    }
  glEnd();
  glPopAttrib();
}

void
OrthoSlicerHandle:: plane_zx(float t)
{
  float zmin = data->region_zmin;
  float zmax = data->region_zmax;
  float xmin = data->region_xmin;
  float xmax = data->region_xmax;
  float dz = (zmax - zmin) / 10;
  float dx = (xmax - xmin) / 10;
  float  y = data->y[cut_plane];

  glPushAttrib(GL_LINE_BIT);
  blink(t);
  glLineWidth(2);
  glBegin(GL_LINES);
    for (int k=0; k<=10; k++) {  
      float z = zmin + dz*k;
      glVertex3f(xmin,y,z);
      glVertex3f(xmax,y,z);
    }
    for (int i=0; i<=10; i++) {  
      float x = xmin + dx*i;
      glVertex3f(x,y,zmin);
      glVertex3f(x,y,zmax);
    }
  glEnd();
  glPopAttrib();
}

void
OrthoSlicerHandle:: initialize()
{
}


void
OrthoSlicerHandle:: wand_motion(int cut_plane0)
{
  if(!on) return; //2008.4.21

  float pos[3];
  float dx, dy, dz;
  int   di, dj, dk;
  int temp_cut_plane; //2008.4.21

  CAVEGetPosition(CAVE_WAND_NAV, pos);
  switch (axis) {
    case I:
      dx = pos[0] - snapstate.wand_pos_x();
      di = int((dx / 2.0) * data->n1);
      temp_cut_plane = cut_plane0 + di;
      if (temp_cut_plane > data->n1-1) temp_cut_plane = data->n1 - 1;
      if (temp_cut_plane < 0) temp_cut_plane = 0;
      cut_plane = temp_cut_plane;
      break;
    case J:
      dy = pos[1] - snapstate.wand_pos_y();
      dj = int((dy / 2.0) * data->n2);
      temp_cut_plane = cut_plane0 + dj;
      if (temp_cut_plane > data->n2-1) temp_cut_plane = data->n2 - 1;
      if (temp_cut_plane < 0) temp_cut_plane = 0;
      cut_plane = temp_cut_plane;
      break;
    case K:
      dz = pos[2] - snapstate.wand_pos_z();
      dk = int((dz / 2.0) * data->n3);
      temp_cut_plane = cut_plane0 + dk;
      if (temp_cut_plane > data->n3-1) temp_cut_plane = data->n3 - 1;
      if (temp_cut_plane < 0) temp_cut_plane = 0;
      cut_plane = temp_cut_plane;
      break;
  }
}


void
OrthoSlicerHandle:: draw()
{
  if (!on) return;
  float x = snapstate.head_pos_x();
  float y = snapstate.head_pos_y();
  float z = snapstate.head_pos_z();
  float distance = 2.0; // feet away
  float phi = snapstate.head_phi();

  x -= distance * sin(phi);   // Since phi=0 is y-axis.
  y += distance * cos(phi);
  z -= 1.0;                   // Shift 0 feet down.
  
  float t = CAVEGetTime();    // Used for animation

  glPushMatrix();
    glTranslatef(x,y,z);
    glScalef(scale, scale, scale);
    glDisable(GL_LIGHTING);
    glCallList(boundaryBox->dislis_no());
    switch (axis) {
      case I:
        plane_yz(t); break;
      case J:
        plane_zx(t); break;
      case K:
        plane_xy(t); break;
    }
  glPopMatrix();
}



void
OrthoSlicerHandle:: snap()
{
  snapstate.record_head_info();  // When middle button is pressed.
  snapstate.record_wand_info();  // Wand is used to shift the cut plane.
}
  
