//
// vfive: Vector Field Interactive Visualization Environment
//
//
// panel.c++
//
//                by Kageyama, Akira  (kage@nifs.ac.jp)  2000.03.17
//
		  
//		  Modified by ohno			2008.1.31
//		  Textures of panels are created by tex_maker in case of  windows
// Please read the file "LICENCE_ja.txt"(in Japanese) or "LICENCE_en.txt" (in English)
// before you use this software. 

#include "vfive.h"

// To seek for label names for vectors and scalars.

extern Menu*      menu;
extern Panel*     pan_main;
extern Panel*     pan_vect_field_choice;
extern Panel*     pan_scal_field_choice;
extern Panel*     pan_vect_exec_kind_choice;
extern Panel*     pan_scal_exec_kind_choice;


int
Panel::is_it_shot(PointerBeam *beam, float pos[])
{
  float to_panel_v[3], foot_pos[3], foot_v[3];
  float scale;

  scale = scaleintp();

  to_panel_v[0] = pos[0] - beam->base[0]* scale;
  to_panel_v[1] = pos[1] - beam->base[1]* scale;
  to_panel_v[2] = pos[2] - beam->base[2]* scale;

  float ip = vec_dotprdct(beam->direct, to_panel_v);

  foot_pos[0] = beam->base[0]* scale + ip*beam->direct[0];
  foot_pos[1] = beam->base[1]* scale + ip*beam->direct[1];
  foot_pos[2] = beam->base[2]* scale + ip*beam->direct[2];

  foot_v[0] = pos[0] - foot_pos[0];
  foot_v[1] = pos[1] - foot_pos[1];
  foot_v[2] = pos[2] - foot_pos[2];

  if( vec_amp(foot_v) <= menu->shoot_ball_rad )
    return(1);
  else
    return(0);
}

void
Panel::check_and_exec_g()
{
  int i = active_row;
  if ( i >= 0 ) {              // There's an active row.
    Panf exec = exec_gp[i];
    if (exec != NULL) exec();  // Execute the selected function.
  }
  active_row = -1;
}

void
Panel::check_and_exec_f(PointerBeam *beam)
{
  for (int i=0; i<nrow; i++) {
    int hit = is_it_shot(beam, pos[i]);
    if (hit) {
      selected_row = active_row = i;
      Panf exec = exec_fp[i];
      if (exec != NULL) exec();  // execute the selected function.
      return;
    }
  }
  active_row = -1;
}


void
Panel::show()
{
  float x = menu->snap_head_pos_x();
  float y = menu->snap_head_pos_y();
  float z = menu->snap_head_pos_z();

  int i, tID;
  float phi = menu->appear_angle_phi - level * menu->dphi;

  for (i=0; i<nrow; i++) {
    pos[i][0] = x - sin(phi) * menu->distance;
    pos[i][1] = y + cos(phi) * menu->distance;
    pos[i][2] = z + menu->top_pos - i * menu->panel_h;
  }

  tID = CAVEUniqueIndex();

  glPushMatrix();
    glTranslatef(x,y,z);
    glRotatef(inl_r2d(phi), 0.0, 0.0, 1.0);
    for (i=0; i<nrow; i++){
      if(i==active_row)
	glCallList(activeBorder_dislis[tID][i]);
      else if(i==selected_row)
	glCallList(selectedBorder_dislis[tID][i]);
      else {
	glCallList(inactiveBorder_dislis[tID][i]);
      }                     
    }

    glColor3f(1.0,1.0,1.0);

    glEnable(GL_TEXTURE_2D);
    for(i=0; i<nrow; i++) {
     float w = menu->image_w;
     float h = menu->image_h;
     float shift_v = menu->top_pos - (i*menu->panel_h);
     glBindTexture(GL_TEXTURE_2D, image_texObj[tID][i]);
     glPushMatrix();
       glTranslatef(0.0, menu->distance + 0.01, shift_v); // behind the waku.
       glBegin(GL_QUADS);
         glTexCoord2f(0.0, 0.0);
         glVertex3f(-w/2, 0.0,  -h/2);
         glTexCoord2f(1.0, 0.0);
         glVertex3f( w/2, 0.0,  -h/2);
         glTexCoord2f(1.0, 1.0);
         glVertex3f( w/2, 0.0, h/2);
         glTexCoord2f(0.0, 1.0);
         glVertex3f(-w/2, 0.0, h/2);
       glEnd();
     glPopMatrix();
    }
    glDisable(GL_TEXTURE_2D);
  glPopMatrix();
}

void
Panel::make_border(int ith)
{
  float h  = menu->panel_h;
  float h2 = menu->image_h / 2;
  float w2 = menu->image_w / 2;

  float shift_v = menu->top_pos - (ith*h);
  glPushMatrix();
    glTranslatef(0.0, menu->distance, shift_v);
    glBegin(GL_LINE_LOOP);
      glVertex3f(-w2, 0.0, h2);
      glVertex3f( w2, 0.0, h2);
      glVertex3f( w2, 0.0,-h2);
      glVertex3f(-w2, 0.0,-h2);
    glEnd();
    glBegin(GL_POINTS);
      glVertex3f(-w2, 0.0, h2);
      glVertex3f( w2, 0.0, h2);
      glVertex3f( w2, 0.0,-h2);
      glVertex3f(-w2, 0.0,-h2);
    glEnd();
  glPopMatrix();
}

void 
Panel::make_activeBorder(int ith, GLuint disl)  
{
  glNewList(disl, GL_COMPILE);
    glPushAttrib(GL_LINE_BIT | GL_POINT_BIT);
      glLineWidth(5);
      glPointSize(5);
      glColor3f(1.0, 0.0, 0.0);
      make_border(ith);
    glPopAttrib();
  glEndList();
}


void 
Panel::make_inactiveBorder(int ith, GLuint disl)  
{
  glNewList(disl, GL_COMPILE);
    glPushAttrib(GL_LINE_BIT | GL_POINT_BIT);
      glLineWidth(5);
      glPointSize(5);
      glColor3f(0.8, 0.8, 0.8);
      make_border(ith);
    glPopAttrib();
  glEndList();
}


void 
Panel::make_selectedBorder(int ith, GLuint disl)  
{
  glNewList(disl, GL_COMPILE);
    glPushAttrib(GL_LINE_BIT | GL_POINT_BIT);
      glLineWidth(5);
      glPointSize(5);
      glColor3f(1.0, 1.0, 0.0);
      make_border(ith);
    glPopAttrib();
  glEndList();
}

void
Panel::make_display_list()
{
  GLuint disl;
  int tID = CAVEUniqueIndex();

  active_row = -1;  // No active panel in the beginning.
  selected_row = -1;

  for (int i=0; i<nrow; i++) {
    disl = inactiveBorder_dislis[tID][i] = glGenLists(1);
    make_inactiveBorder(i, disl);

    disl = activeBorder_dislis[tID][i] = glGenLists(1);
    make_activeBorder(i, disl);

    disl = selectedBorder_dislis[tID][i] = glGenLists(1);
    make_selectedBorder(i, disl);
  }
}
  

void
Panel::make(int level_,
	    char* string0, Panf f0, Panf g0,
	    char* string1, Panf f1, Panf g1, 
	    char* string2, Panf f2, Panf g2,
	    char* string3, Panf f3, Panf g3, 
	    char* string4, Panf f4, Panf g4,
	    char* string5, Panf f5, Panf g5, 
	    char* string6, Panf f6, Panf g6,
	    char* string7, Panf f7, Panf g7, 
	    char* string8, Panf f8, Panf g8,
	    char* string9, Panf f9, Panf g9)
{
  int i;

  level = level_;

  strcpy(string[0], string0);  strcpy(string[1], string1);
  strcpy(string[2], string2);  strcpy(string[3], string3);
  strcpy(string[4], string4);  strcpy(string[5], string5);
  strcpy(string[6], string6);  strcpy(string[7], string7);
  strcpy(string[8], string8);  strcpy(string[9], string9);

  exec_fp[0] = f0;  exec_fp[1] = f1;  exec_fp[2] = f2;
  exec_fp[3] = f3;  exec_fp[4] = f4;  exec_fp[5] = f5;
  exec_fp[6] = f6;  exec_fp[7] = f7;  exec_fp[8] = f8;
  exec_fp[9] = f9;

  exec_gp[0] = g0;  exec_gp[1] = g1;  exec_gp[2] = g2;
  exec_gp[3] = g3;  exec_gp[4] = g4;  exec_gp[5] = g5;
  exec_gp[6] = g6;  exec_gp[7] = g7;  exec_gp[8] = g8;
  exec_gp[9] = g9;

  int ctr = 0;
  while( strcmp(string[ctr++],"") != 0 );  // counting the number

  nrow = ctr-1;
  printf(" nrow = %d\n", nrow);

  for (i=0; i<nrow; i++) {
    printf("  string%d = %s\n", i, string[i]);
  }

}

void
Panel::make2(int level_, int n, char* base, Panf f) 
{

  int i;
  char st[50];

  level = level_;
  for (i=0; i<n; i++) {
    sprintf(st,"%s%02d", base, i);  // "Vect_" + "01"
    strcpy(string[i], st);
    exec_fp[i] = f;
    exec_gp[i] = NULL;
  }
  for (i=n; i<10; i++) {
    strcpy(string[i], "");
    exec_fp[i] = NULL;
    exec_gp[i] = NULL;
  }

  nrow = n;
  printf(" nrow = %d\n", nrow);

  for (i=0; i<nrow; i++) {
    printf("  string%d = %s\n", i, string[i]);
  }

}

static 
void parse(char *string1, char *string2)
{
    char *p, *q;

    q = string1;
    *q ='\0';
    for (p = string2; *p != '\0'; p++) {

	*q = *p;
	
	if (*p == '\\' && *(p + 1) == 'n') {
	q++;
	*q='\\';
	}

	q++;
	*q ='\0';
    }
}

void
Panel::make_image(char *label, unsigned long bgc,
unsigned long fgc, char *file_name)
{
	char label_temp[60];

	parse(label_temp, label);

	char command[256];
#ifdef WIN32
	sprintf(command,"tex_maker %s 0x%x 0x%x %s",
	 label_temp,bgc,fgc,file_name);
#else
	sprintf(command,"./tex_maker %s 0x%x 0x%x %s",
	 label_temp,bgc,fgc,file_name);
#endif
	system(command);
}

void
Panel::set_image(int ith, char *file_name)
{
	unsigned char image[PANEL_IMAGE_H][PANEL_IMAGE_W][3];
        int tID;
	FILE *fp;

        if ((fp = fopen(file_name, "rb")) == NULL) {
	 printf("File Open Error:%s\n",file_name);
	 exit(0);
	 }

	  fread(&image[0][0][0], sizeof(unsigned char),
		   PANEL_IMAGE_W * PANEL_IMAGE_H * 3, fp);

	fclose(fp);

	// create Texture Objext
        tID = CAVEUniqueIndex();
        glGenTextures(1, &image_texObj[tID][ith]);

        glBindTexture(GL_TEXTURE_2D, image_texObj[tID][ith]);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
        glTexImage2D(GL_TEXTURE_2D, 0, 3, PANEL_IMAGE_W,
	           PANEL_IMAGE_H, 0, GL_RGB, GL_UNSIGNED_BYTE, 
	           &image[0][0][0]);

}
