//
// vfive: Vector Field Interactive Visualization Environment
//
// pointerbeam.c++
//
// pointer beam functions
//
//                by Kageyama, Akira  (kage@nifs.ac.jp)  2000.03.15
//  
// Please read the file "LICENCE_ja.txt"(in Japanese) or "LICENCE_en.txt" (in English)
// before you use this software. 

#include "vfive.h"

PointerBeam::PointerBeam(float r, float g, float b, float leng)   // Constructor
{
  on = 0;                    // default on
  length = leng;             // feet
  beamcolor[0] = r;          // white
  beamcolor[1] = g;
  beamcolor[2] = b;
  line_width = 4.0;          // pixels

  for(int i=0;i<3;i++){
   base[i]   = 0.0;           // base point position
   tip[i]    = 0.0;            // tip point position 
   direct[i] = 0.0;         // unit vector of wand direction
  }

}

void 
PointerBeam::drawLine(float vert1[], float vert2[])
{
  glBegin(GL_LINES);
    glVertex3fv(vert1);
    glVertex3fv(vert2);
  glEnd();
}

void 
PointerBeam::reset()
{
  float   wand_pos[3];
  float   scale;

  scale = scaleintp();

  CAVEGetPosition(CAVE_WAND_NAV, wand_pos);
  CAVEGetVector(CAVE_WAND_FRONT_NAV, direct);

  for (int i=0; i<3; i++) {
    base[i] = wand_pos[i]/scale;

    if(CAVEBUTTON1) tip[i] = wand_pos[i]/scale + direct[i] * length/scale;
    else tip[i] = wand_pos[i]/scale + direct[i] * length;
  }

}


void
PointerBeam::draw()
{
  if (!on) return;
  glPushAttrib(GL_LIGHTING_BIT | GL_LINE_BIT);
  glDisable(GL_LIGHTING);
  glColor3fv(beamcolor);
  glLineWidth(line_width);
  drawLine(base, tip);
  glPopAttrib();
}
