//
// vfive: Vector Field Interactive Visualization Environment
//
// probe.c++
//
//                by Nobuaki Ohno               2008.10.
//
// Please read the file "LICENCE_ja.txt"(in Japanese) or "LICENCE_en.txt" (in English)
// before you use this software. 

#include "vfive.h"
extern Data *data;
extern cave_message *cave_mssg;
extern char*          inputFile;           // vfive input data.

static 
void parse(char *string1, char *string2)
{
    char *p, *q;

    q = string1;
    *q ='\0';
    for (p = string2; *p != '\0'; p++) {

	if (*p == '\\' && *(p + 1) == 'n') {
	*q=' ';
	 p++;
	} else {
	*q = *p;
	}
	q++;
	*q ='\0';
    }
}


Probe::Probe(void)
{
  on = 0;
  chosen_var = -1;
  for(int i=0;i<MAX_NVEC;i++){
   v_on[i] = 0;
   vval[i][0] = 0.0;
   vval[i][1] = 0.0;
   vval[i][2] = 0.0;
   vval[i][3] = 0.0;
   v_outaregion[i] = 1;
   }

  for(int i=0;i<MAX_NSCAL;i++){
   s_on[i] = 0;
   sval[i] = 0.0;
   s_outaregion[i] = 1;
  }

  beam = new PointerBeam(0.3, 1.0, 1.0, 3.0);
  if(beam == NULL) v5_error("can't allocate beam: Probe");

  snapstate = new Snapstate;
  if(snapstate == NULL) v5_error("can't allocate snapstate: Probe");
 
  for (int i=0; i<data->nscal; i++) {
    char key[20], label[50];
    sprintf(key, "scal%d_label", i);
    v5_get_string(inputFile, key, label);  
    parse(scal_name[i], label);   
  }

  for (int i=0; i<data->nvec; i++) {
    char key[20], label[50];
    sprintf(key, "vect%d_label", i);   
    v5_get_string(inputFile, key, label);  
    parse(vec_name[i], label);   
  }

}

void Probe::probe_calc()
{
   if(!on) return;

    snapstate->record_head_info();

    int cv = chosen_var;
    ffloat_ val;
    ffloat_ vec[3];
    
    if(cv < 0) return;
      else {

    if(cv >= MAX_NVEC){ //one of scalar is chosen
	 cv -= MAX_NVEC;
	 spos[cv][0] = beam->tip[0];
	 spos[cv][1] = beam->tip[1];
	 spos[cv][2] = beam->tip[2];
	 if(intpol_get_scalar(cv, spos[cv], &val)){
	  s_outaregion[cv] = 1;
	  return;
	 }
	 sval[cv] = val;
	 s_outaregion[cv] = 0;

     } else { //one of vector is chosen
	 vpos[cv][0] = beam->tip[0];
	 vpos[cv][1] = beam->tip[1];
	 vpos[cv][2] = beam->tip[2];
	 if(intpol_get_vector(cv, vpos[cv], vec, &val)){
	  v_outaregion[cv] = 1;
	  return;
	 }
	 vval[cv][0] = val;
	 vval[cv][1] = vec[0];
	 vval[cv][2] = vec[1];
	 vval[cv][3] = vec[2];
	 v_outaregion[cv] = 0;

     }

    }     
}

static void star(ffloat_ v, ffloat_ vmax, ffloat_ vmin){

  float r, g, b;
  float normed_v;// = (vmax-v)/(vmax-vmin);
  normed_v = (vmax-v)/(vmax-vmin);

  float hue;// = 0.7 * normed_v;
  hue = 0.7 * normed_v;
  float sat = 1.0;
  float val = 1.0;

  hsv2rgb(hue, sat, val, &r, &g, &b);

  glPushAttrib(GL_LIGHTING_BIT | GL_LINE_BIT);
  glColor3f(r, g, b);
   glLineWidth(5.0);
    glBegin(GL_LINES);
     glVertex3f(0.0, 0.0, -0.1);
     glVertex3f(0.0, 0.0,  0.1);

     glVertex3f(-0.1, 0.0, 0.0);
     glVertex3f( 0.1, 0.0, 0.0);

     glVertex3f(0.0, -0.1, 0.0);
     glVertex3f(0.0,  0.1, 0.0);
    glEnd();
  glPopAttrib();

}

static void pin(ffloat_ *vect){
  float tip[3], base[3];
  float mag;
  mag = sqrt(vect[1]*vect[1] + vect[2]*vect[2] + vect[3]*vect[3])*5.0;

  //0.5 feet
  tip[0] = vect[1]/mag;
  tip[1] = vect[2]/mag;
  tip[2] = vect[3]/mag;

  base[0] = -vect[1]/mag;
  base[1] = -vect[2]/mag;
  base[2] = -vect[3]/mag;
  
  glPushAttrib(GL_LIGHTING_BIT | GL_LINE_BIT | GL_POINT_BIT);
  glColor3f(0.4, 1.0, 1.0);
   glLineWidth(3.0);
    glBegin(GL_LINES);
     glVertex3fv(base);
     glVertex3fv(tip);
    glEnd();

   glPointSize(10.0);
    glBegin(GL_POINTS);
     glVertex3fv(base);
    glEnd();
   glPointSize(1);
  glPopAttrib();
}

void Probe::draw(void)
{
    if(!on) return;

    float phi = snapstate->head_phi();
    char mssg[200];

    float vspace;
    float space;
    glPushAttrib(GL_LIGHTING_BIT);
    glDisable(GL_LIGHTING);

    beam->draw();

    for(int i=0;i<MAX_NVEC;i++){
    vspace = 0.0;
    space = 0.0;
     if(i >= data->nvec) break;

     if(!v_on[i] || v_outaregion[i]) continue;

      glPushMatrix();

      glTranslatef(vpos[i][0], vpos[i][1], vpos[i][2]);
      star(vval[i][0], data->vector_fld[i]->vmax,
	data->vector_fld[i]->vmin);
      pin(vval[i]);
      glTranslatef(0.0, 0.0, -0.325);
      // Rotation to face the viewer
      glRotatef(phi * 180.0 / PI, 0.0, 0.0, 1.0);

      glColor3f(1.0, 1.0, 1.0);
      sprintf(mssg, "%f ", vval[i][0]);
      cave_mssg->write_message(0.2, mssg, &space, &vspace);

      glColor3f(1.0, 1.0, 0.4);
      sprintf(mssg, "(%s)\n", vec_name[i]);
      cave_mssg->write_message(0.2, mssg, &space, &vspace);

      glColor3f(1.0, 1.0, 1.0);
      space = 0.0;
      sprintf(mssg, "X: %f\n", vval[i][1]);
      cave_mssg->write_message(0.2, mssg, &space, &vspace);

      space = 0.0;
      sprintf(mssg, "Y: %f\n", vval[i][2]);
      cave_mssg->write_message(0.2, mssg, &space, &vspace);

      space = 0.0;
      sprintf(mssg, "Z: %f\n", vval[i][3]);
      cave_mssg->write_message(0.2, mssg, &space, &vspace);

      glPopMatrix();
    }

    for(int i=0;i<MAX_NSCAL;i++){
     vspace = 0.0;
     space = 0.0;
     if(i >= data->nscal) break;
     if(!s_on[i] || s_outaregion[i]) continue;
      glPushMatrix();

      glTranslatef(spos[i][0], spos[i][1], spos[i][2]);
      star(sval[i], data->scalar_fld[i]->vmax,
	data->scalar_fld[i]->vmin);
      glTranslatef(0.0, 0.0, -0.325);
      // Rotation to face the viewer
      glRotatef(phi * 180.0 / PI, 0.0, 0.0, 1.0);

      glColor3f(1.0, 1.0, 1.0);
      sprintf(mssg, "%f ", sval[i]);
      cave_mssg->write_message(0.2, mssg, &space, &vspace);

      glColor3f(1.0, 1.0, 0.4);
      sprintf(mssg, "(%s)\n", scal_name[i]);
      cave_mssg->write_message(0.2, mssg, &space, &vspace);

      glPopMatrix();
    }

    glPopAttrib();

}

void
Probe::button_being_pressed()
{
  if(!on) return;
  beam->reset();
}

void
Probe::button_has_pressed()
{
  beam->reset();
  beam->turn(1);

  if(chosen_var < 0) return;
   else if(chosen_var < MAX_NVEC){
    v_on[chosen_var] = 1;
   }
   else {
    s_on[chosen_var - MAX_NVEC] = 1;
   }
}

void
Probe::button3_has_pressed()
{
  if(chosen_var < 0) return;
   else if(chosen_var < MAX_NVEC){
    v_on[chosen_var] = 0;
   }
   else {
    s_on[chosen_var - MAX_NVEC] = 0;
   }

  int t = 0;
  for(int i=0;i<data->nvec;i++) t += v_on[i];
  for(int i=0;i<data->nscal;i++) t += s_on[i];

  if(t == 0) on = 0;

  chosen_var = -1;
}


void
Probe::button_has_released()
{
  beam->turn(0);
}

