//
// vfive: Vector Field Interactive Visualization Environment
//
//
// surface_obj.c++
//
//                by Kageyama, Akira  (kage@nifs.ac.jp)  2000.04.12
//  
// Please read the file "LICENCE_ja.txt"(in Japanese) or "LICENCE_en.txt" (in English)
// before you use this software. 


#include "vfive.h"

const float BIG_VALUE_FOR_EDGE_CHECK = 999.;

extern Data* data;

static void 
read_data(char *file, float *a, int size)  
{
  FILE *fp;
  char dummy[4];
  
   fp = fopen(file,"rb");
   if (fp==NULL) v5_error("fopen failed in read_data, surface_obj.c++");
   fread(dummy, 1, 4, fp); // skip the first 4 byte in a fortran record
   fread((char *)a, 1, size*sizeof(float), fp);
   fclose(fp);
}

int
SurfaceObj:: find_ni(int n)
{
  for (int i=0; i<ni_max; i++) {
    float x_ = x[n][0][i];
    float y_ = y[n][0][i];
    float z_ = z[n][0][i];
    if (    x_ > BIG_VALUE_FOR_EDGE_CHECK 
	 && y_ > BIG_VALUE_FOR_EDGE_CHECK 
	 && z_ > BIG_VALUE_FOR_EDGE_CHECK ) return i;
  }
  return (ni_max);
}

int
SurfaceObj:: find_nj(int n)
{
  for (int j=0; j<nj_max; j++) {
    float x_ = x[n][j][0];
    float y_ = y[n][j][0];
    float z_ = z[n][j][0];
    if ( abs(x_) > 200.0 ) 
      printf("   x_ y,z = %f %f %f\n", x_, y_, z_);

    if (    x_ > BIG_VALUE_FOR_EDGE_CHECK 
	 && y_ > BIG_VALUE_FOR_EDGE_CHECK 
	 && z_ > BIG_VALUE_FOR_EDGE_CHECK ) return j;
  }
  return (nj_max);
}


void
SurfaceObj:: read()
{
  char xfile[200], yfile[200], zfile[200];
  int n;

   v5_get_string(inputFile, "surfaceObj:xfile", xfile);
   v5_get_string(inputFile, "surfaceObj:yfile", yfile);
   v5_get_string(inputFile, "surfaceObj:zfile", zfile);

   // check the results 
   printf("v5: surfaceObj: nmen    = %d\n", nmen);
   printf("v5: surfaceObj: ni_max = %d\n", ni_max);
   printf("v5: surfaceObj: nj_max = %d\n", nj_max);
   printf("v5: surfaceObj: xfile  = %s\n", xfile);
   printf("v5: surfaceObj: yfile  = %s\n", yfile);
   printf("v5: surfaceObj: zfile  = %s\n", zfile);

  // data rading 
  read_data(xfile, &(x[0][0][0]), nmen*ni_max*nj_max);
  read_data(yfile, &(y[0][0][0]), nmen*ni_max*nj_max);
  read_data(zfile, &(z[0][0][0]), nmen*ni_max*nj_max);

  // count mesh size of each surface.
  for (n=0; n<nmen; n++) {
    ni[n] = find_ni(n);
    nj[n] = find_nj(n);
    printf(" ni[%d] = %d   nj[%d] = %d \n", n, ni[n], n, nj[n]);
  }

  const float feet_per_meter = 3.280840;
  float norms = feet_per_meter * (data->scale);
  float shift_x = data->shift_x;
  float shift_y = data->shift_y;
  float shift_z = data->shift_z;

  for (n=0; n<nmen; n++) {
    for (int i=0; i<ni[n]; i++) {
      for (int j=0; j<nj[n]; j++) {
	x[n][j][i] += shift_x;        // coordinate shift 
	y[n][j][i] += shift_y;
	z[n][j][i] += shift_z;
	x[n][j][i] *= norms;          // scale transform
	y[n][j][i] *= norms;          
	z[n][j][i] *= norms;          
      }
    }
  }
}


void
SurfaceObj:: make(char* file)
{
   inputFile = file;
   v5_get_int(file, "surfaceObj:nmen", &nmen);
   v5_get_int(file, "surfaceObj:ni_max", &ni_max);
   v5_get_int(file, "surfaceObj:nj_max", &nj_max);

  ni = create_array_i1(nmen);
  nj = create_array_i1(nmen);
  x = create_array_f3(nmen, nj_max, ni_max);   // always single precision.
  y = create_array_f3(nmen, nj_max, ni_max);
  z = create_array_f3(nmen, nj_max, ni_max);
  nx = create_array_f3(nmen, nj_max, ni_max);
  ny = create_array_f3(nmen, nj_max, ni_max);
  nz = create_array_f3(nmen, nj_max, ni_max);
  turn(0);                 // default off
}

int 
SurfaceObj:: check_topology_in_i(int n) {
  int nim1 = ni[n] - 1 ;
  for (int j=0; j<nj[n]; j++) {
    float x_stt = x[n][j][0];
    float y_stt = y[n][j][0];
    float z_stt = z[n][j][0];
    float x_end = x[n][j][nim1];
    float y_end = y[n][j][nim1];
    float z_end = z[n][j][nim1];
    if ( x_stt != x_end || y_stt != y_end || z_stt != z_end ) 
      return 0;  // This isn't periodic in i-direction.
  }
  return 1;
}
int 
SurfaceObj:: check_topology_in_j(int n) {
  int njm1 = nj[n] - 1;
  for (int i=0; i<ni[n]; i++) {
    float x_stt = x[n][0][i];
    float y_stt = y[n][0][i];
    float z_stt = z[n][0][i];
    float x_end = x[n][njm1][i];
    float y_end = y[n][njm1][i];
    float z_end = z[n][njm1][i];
    if ( x_stt != x_end || y_stt != y_end || z_stt != z_end ) 
      return 0;  // This isn't periodic in j-direction.
  }
  return 2;
}

char
SurfaceObj:: check_topology(int n)
{
  char c;
  int i = check_topology_in_i(n) + check_topology_in_j(n);
  switch (i) {
    case 0 : c = '0'; break;   // No periodicity
    case 1 : c = 'i'; break;   // Periodic in i-direction.
    case 2 : c = 'j'; break;   // Periodic in j-direction.
    case 3 : c = 't'; break;   // Torus topology
    default :
      v5_error("Someting is wrong in check_topology.");
  }
  return c;
}

static int
ip1_(int i, int ni, char topo)
{                       //           0 1 2 3 4 5
  int ip1 = i+1;        // 0 1 2 3 4 5     in case of ni=6
  if (i==ni-1) {
    switch (topo) {
      case 'i' :   // periodic in i-direction
      case 't' :   // also in the torus case.
        ip1 = 1; 
        break;
      default :
        ip1 = ni-1;
    }
  }
  return ip1;
}

static int
im1_(int i, int ni, char topo)
{                       //           0 1 2 3 4 5
  int im1 = i-1;        // 0 1 2 3 4 5     in case of ni=6
  if (i==0) {
    switch (topo) {
      case 'i' :   // periodic in i-direction
      case 't' :   // also in the torus case.
        im1 = ni-2; 
        break;
      default :
        im1 = 0;
    }
  }
  return im1;
}

static int
jp1_(int j, int nj, char topo)
{                       //           0 1 2 3 4 5
  int jp1 = j+1;        // 0 1 2 3 4 5     in case of nj=6
  if (j==nj-1) {
    switch (topo) {
      case 'j' :   // periodic in i-direction
      case 't' :   // also in the torus case.
        jp1 = 1; 
        break;
      default :
        jp1 = nj-1;
    }
  }
  return jp1;
}

static int
jm1_(int j, int nj, char topo)
{                       //           0 1 2 3 4 5
  int jm1 = j-1;        // 0 1 2 3 4 5     in case of nj=6
  if (j==0) {
    switch (topo) {
      case 'j' :   // periodic in i-direction
      case 't' :   // also in the torus case.
        jm1 = nj-2; 
        break;
      default :
        jm1 = 0;
    }
  }
  return jm1;
}

void
SurfaceObj:: make_normal(int n, char topo)
{
  float vect_i[3], vect_j[3], vect_n[3];

  for (int i=0; i<ni[n]; i++) {
    int ip1 = ip1_(i, ni[n], topo);
    int im1 = im1_(i, ni[n], topo);
    for (int j=0; j<nj[n]; j++) {
      int jp1 = jp1_(j, nj[n], topo);
      int jm1 = jm1_(j, nj[n], topo);
      vect_i[0] = x[n][j][ip1] - x[n][j][im1];
      vect_i[1] = y[n][j][ip1] - y[n][j][im1];
      vect_i[2] = z[n][j][ip1] - z[n][j][im1];
      vect_j[0] = x[n][jp1][i] - x[n][jm1][i];
      vect_j[1] = y[n][jp1][i] - y[n][jm1][i];
      vect_j[2] = z[n][jp1][i] - z[n][jm1][i];
      vec_cross_normalf(vect_i, vect_j, vect_n);
      nx[n][j][i] = vect_n[0];
      ny[n][j][i] = vect_n[1];
      nz[n][j][i] = vect_n[2];
    }
  }
}

void
SurfaceObj:: triangle_strip(int n)
{
  for (int i=0; i<ni[n]-1; i++) {
    glBegin(GL_TRIANGLE_STRIP);
    for (int j=0; j<nj[n]; j++) {
      glNormal3f(nx[n][j][i], ny[n][j][i], nz[n][j][i]);
      glVertex3f( x[n][j][i],  y[n][j][i],  z[n][j][i]);
      glNormal3f(nx[n][j][i+1], ny[n][j][i+1], nz[n][j][i+1]);
      glVertex3f( x[n][j][i+1],  y[n][j][i+1],  z[n][j][i+1]);
    }
    glEnd();
  }      
}

void
SurfaceObj:: construct_each_surface(int n)
{
  char topo = check_topology(n);
  make_normal(n, topo);
  triangle_strip(n);
}


void
SurfaceObj:: initialize()
{
  GLfloat diffuse[]  = {0.8, 0.9, 0.3, 1.0};
  GLfloat ambient[]  = {0.3, 0.3, 0.0, 1.0};
  GLfloat specular[] = {0.3, 0.6, 0.3, 1.0};
  int tID = CAVEUniqueIndex();

  dislis[tID] = glGenLists(1);
  glNewList(dislis[tID], GL_COMPILE);
    glEnable(GL_LIGHTING);
    glMaterialfv(GL_FRONT, GL_DIFFUSE,  diffuse);
    glMaterialfv(GL_FRONT, GL_AMBIENT,  ambient);
    glMaterialfv(GL_FRONT, GL_SPECULAR, specular);
    glMaterialf (GL_FRONT, GL_SHININESS, 10.0);
    for (int n=0; n<nmen; n++)  construct_each_surface(n);
  glEndList();
}

void
SurfaceObj:: draw()
{
  if (on==0) return;
  glPushAttrib(GL_LIGHTING_BIT);
  glCallList(dislis[CAVEUniqueIndex()]);
  glPopAttrib();
}

