//
// vfive: Vector Field Interactive Visualization Environment
//
//                           ver.3.01
//
//                by Kageyama, Akira  (kage@nifs.ac.jp)  2000.03.17
//
// main.c++
//  
// 
//          Akira Kageyama,
//          Theory and Computer Simulation Center.
//          National Institute for Fusion Science.
//          kage@nifs.ac.jp
// 
//
// Please read the file "LICENCE_ja.txt"(in Japanese) or "LICENCE_en.txt" (in English)
// before you use this software. 

#include "vfive.h"

extern Target*       target;
extern Menu*         menu;
extern Data*         data;
extern Tracer*       tracer;
extern LocalArrows*  localArrows;
extern Isosurf*      isosurf;
extern CurveObj*     curveObj;
extern SurfaceObj*   surfaceObj;
extern OrthoSlicer*  orthoSlicer[3];          // xy, yz, and zx planes.
extern LocalSlicer*  localSlicer;
extern BoundaryBox*  boundaryBox;
extern Snow*         snow;
extern Hotaru*       hotaru;
extern Snapshot*     snapshot;
extern cave_volrend*     volumerendering;
extern scalefHandle*     scalefh;
extern Probe*     probe;

extern int recalc_flag[CAVE_MAX_WALLS][5]; // 0:iso, 1:ortho_i, 2:ortho_j, 3:ortho_k 4:volume rendering

///////////////////////////////////
//   BUTTON 1
///////////////////////////////////

void 
button1()
{
  if (CAVEBUTTON1) 
    menu->button_being_pressed();

  int bc = CAVEButtonChange(1);

  if (bc == 1) {
    menu->button_has_pressed();
  }
  else if (bc == -1) {
    menu->button_has_released();
  }
}

///////////////////////////////////
//   BUTTON 2
///////////////////////////////////

void 
button2()
{
  viz_kind viz = target->viz();

  if (CAVEBUTTON2) {
    switch (viz) {
    case ISOSURF:
      isosurf->button_being_pressed();
      break;
    case TRACER:
      tracer->button_being_pressed();
      break;
    case ORTHO_SLICER_I:
      orthoSlicer[0]->button_being_pressed();
      break;
    case ORTHO_SLICER_J:
      orthoSlicer[1]->button_being_pressed();
      break;
    case ORTHO_SLICER_K:
      orthoSlicer[2]->button_being_pressed();
      break;
    case LOCAL_ARROWS:
      localArrows->button_being_pressed();
      break;
    case SNOW:
      snow->button_being_pressed();
      break;
    case LOCAL_SLICER:
      localSlicer->button_being_pressed();
      break;
    case SCALE_HANDLE:
      scalefh->button_being_pressed();
      break;
    case VOLUME_RENDER:
      volumerendering->button_being_pressed();
      break;
    case PROBE:
      probe->button_being_pressed();
      break;
    }
  }

  int bc = CAVEButtonChange(2);

  if (bc == 1) {
    switch (viz) {
    case LOCAL_ARROWS:
      localArrows->button_has_pressed();
      break;
    case TRACER:
      tracer->button_has_pressed();
      break;
    case ISOSURF:
      isosurf->button_has_pressed();
      break;
    case ORTHO_SLICER_I:
      orthoSlicer[0]->button_has_pressed();
      break;
    case ORTHO_SLICER_J:
      orthoSlicer[1]->button_has_pressed();
      break;
    case ORTHO_SLICER_K:
      orthoSlicer[2]->button_has_pressed();
      break;
    case SNOW:
      snow->button_has_pressed();
      break;
    case SNAPSHOT:
      snapshot->button_has_pressed();
      break;
    case LOCAL_SLICER:
      localSlicer->button_has_pressed();
      break;
    case SCALE_HANDLE:
      scalefh->button_has_pressed();
      break;
    case PROBE:
      probe->button_has_pressed();
      break;

    }
  }
  else if (bc == -1) {
    switch (viz) {
    case ISOSURF:
      isosurf->button_has_released();
    for(int i=0;i<CAVENumPipes();i++)
	recalc_flag[i][0] = 1;
      break;
    case LOCAL_ARROWS:
      localArrows->button_has_released();
      break;
    case TRACER:
      tracer->button_has_released();
      break;
    case ORTHO_SLICER_I:
      orthoSlicer[0]->button_has_released();
    for(int i=0;i<CAVENumPipes();i++)
	recalc_flag[i][1] = 1;
      break;
    case ORTHO_SLICER_J:
      orthoSlicer[1]->button_has_released();
    for(int i=0;i<CAVENumPipes();i++)
	recalc_flag[i][2] = 1;
      break;
    case ORTHO_SLICER_K:
      orthoSlicer[2]->button_has_released();
    for(int i=0;i<CAVENumPipes();i++)
	recalc_flag[i][3] = 1;
      break;
    case SNOW:
      snow->button_has_released();
      break;
    case LOCAL_SLICER:
      localSlicer->button_has_released();
      break;
    case SCALE_HANDLE:
      scalefh->button_has_released();
      break;
    case VOLUME_RENDER:
      volumerendering->button2_has_released();
    if(volumerendering->get_recalc_flag()){
      volumerendering->remake_voldata(volumerendering->sscal());
    for(int i=0;i<CAVENumPipes();i++)
	recalc_flag[i][4] = 1;
    }
      break;
    case PROBE:
      probe->button_has_released();
      break;

    }
  }
}


///////////////////////////////////
//   BUTTON 3
///////////////////////////////////

void 
button3()
{
  viz_kind viz = target->viz();

  if (CAVEBUTTON3) {
    // Nothing...
  }

  int bc = CAVEButtonChange(3);

  if (bc == 1) {
    switch (viz) {
    case LOCAL_ARROWS:
      localArrows->button3_has_pressed();
      break;
    case ISOSURF:
      isosurf->button3_has_pressed();
      break;
    case ORTHO_SLICER_I:
      orthoSlicer[0]->button3_has_pressed();
      break;
    case ORTHO_SLICER_J:
      orthoSlicer[1]->button3_has_pressed();
      break;
    case ORTHO_SLICER_K:
      orthoSlicer[2]->button3_has_pressed();
      break;
    case SNOW:
      snow->button3_has_pressed();
      break;
    case HOTARU:
      hotaru->button3_has_pressed();
      break;
    case TRACER:
      tracer->button3_has_pressed();
      break;
    case LOCAL_SLICER:
      localSlicer->button3_has_pressed();
      break;
    case SCALE_HANDLE:
      scalefh->button3_has_pressed();
      break;
    case VOLUME_RENDER:
      volumerendering->button3_has_pressed();
      break;
    case PROBE:
      probe->button3_has_pressed();
      break;

    }
  }
  else if (bc == -1) {
    // Nothing.
  }
}

